/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.business;

import fr.paris.lutece.plugins.stock.service.StockPlugin;
import fr.paris.lutece.portal.service.jpa.JPALuteceDAO;

import org.apache.commons.lang.StringUtils;

import java.lang.reflect.ParameterizedType;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;


/**
 * This class provides Data Access methods for Person objects
 * @param <F> the product filter
 * @param <K> the primary key
 * @param <P> the product
 * @param <P_> the product metamodel
 */
public class AbstractProductDAO<K, P extends Product, F extends ProductFilter, P_ extends Product_> extends JPALuteceDAO<K, P>
    implements IProductDAO<K, P, F>
{
    private Class<P> _entityClass;

    /**
     * Constructor
     */
    public AbstractProductDAO(  )
    {
        _entityClass = (Class<P>) ( (ParameterizedType) getClass(  ).getGenericSuperclass(  ) ).getActualTypeArguments(  )[1];
    }

    /**
     *
     *{@inheritDoc}
     */
    @Override
    public String getPluginName(  )
    {
        return StockPlugin.PLUGIN_NAME;
    }

    /**
     * Get the product P class
     * @return the class
     */
    public Class<P> getProductClass(  )
    {
        return _entityClass;
    }

    /**
     * Find product list by filter
     * @param filter the filter
     * @return the list of product
     */
    public List<P> findByFilter( F filter )
    {
        EntityManager em = getEM(  );
        CriteriaBuilder cb = em.getCriteriaBuilder(  );

        CriteriaQuery<P> cq = cb.createQuery( getProductClass(  ) );

        Root<P> root = cq.from( getProductClass(  ) );

        List<Predicate> listPredicates = buildPredicates( filter, root, cb );
        buildCriteriaQuery( listPredicates, cq );
        buildSortQuery( filter, root, cq, cb );

        TypedQuery<P> query = em.createQuery( cq );

        return query.getResultList(  );
    }

    /**
     * Build the predicate list from filter
     * @param filter the filter
     * @param root the product root
     * @param builder the criteria builder
     * @return the predicate list
     */
    protected List<Predicate> buildPredicates( F filter, Root<P> root, CriteriaBuilder builder )
    {
        // predicates list
        List<Predicate> listPredicates = new ArrayList<Predicate>(  );

        if ( filter.getIdProduct(  ) != null )
        {
            listPredicates.add( builder.equal( root.get( P_.idProduct ), filter.getIdProduct(  ) ) );
        }

        if ( filter.getPrice(  ) != null )
        {
            listPredicates.add( builder.equal( root.get( P_.price ), filter.getPrice(  ) ) );
        }

        if ( filter.getStockQuantity(  ) != null )
        {
            listPredicates.add( builder.equal( root.get( P_.stockQuantity ), filter.getStockQuantity(  ) ) );
        }

        return listPredicates;
    }

    /**
     * Build criteria query from predicates list
     * @param listPredicates the list of predicates
     * @param query the criteria query
     */
    private void buildCriteriaQuery( List<Predicate> listPredicates, CriteriaQuery<P> query )
    {
        if ( !listPredicates.isEmpty(  ) )
        {
            // add existing predicates to Where clause
            query.where( listPredicates.toArray( new Predicate[0] ) );
        }
    }

    /**
     * Build the sort query
     * @param filter the filter
     * @param root the product root
     * @param query the criteria query
     * @param builder the criteria builder
     */
    protected void buildSortQuery( F filter, Root<P> root, CriteriaQuery<P> query, CriteriaBuilder builder )
    {
        if ( StringUtils.isNotBlank( filter.getOrder(  ) ) )
        {
            if ( filter.getOrder(  ) != null )
            {
                Order order = null;

                if ( filter.isOrderAsc(  ) )
                {
                    // get asc order
                    order = builder.asc( root.get( filter.getOrder(  ) ) );
                }
                else
                {
                    // get desc order
                    order = builder.desc( root.get( filter.getOrder(  ) ) );
                }

                query.orderBy( order );
            }
        }
    }
}
