/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.business;

import fr.paris.lutece.plugins.stock.service.GlobalCategoryService;
import fr.paris.lutece.plugins.stock.utils.jpa.StockJPAUtils;

import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.OrderColumn;
import javax.persistence.Table;
import javax.persistence.TableGenerator;
import javax.persistence.Transient;


/**
 *
 * Category
 *
 */
@Entity
@Inheritance( strategy = InheritanceType.JOINED )
@Table( name = "stock_category" )
public class Category
{
    public static final String ATTRIBUTE_CHILDREN_LIST = "childrenList";
    public static final String ATTRIBUTE_PARENT = "parent";

    /** Sequence name */
    private static final String JPA_SEQUENCE_NAME = "stock_category_sequence";

    /** Unique value */
    private static final String JPA_COLUMN_NAME = "stock_category_id";
    private int _idCategory;
    private boolean _status;
    private String _strName;
    private List<Category> _childrenList;
    private List<Product> _productSet;
    private Category _parentCategory;
    private Provider _provider;

    /**
     * Constructor
     */
    public Category(  )
    {
        super(  );
    }

    /**
     * Build a new category from a category object
     * @param category the category
     */
    public Category( Category category )
    {
        _idCategory = category.getIdCategory(  );
        _status = category.getActive(  );
        _strName = category.getName(  );
        _childrenList = category.getChildrenList(  );
        _productSet = category.getProductSet(  );
        _parentCategory = category.getParent(  );
    }

    /**
     * Return category id
     * @return category id
     */
    @TableGenerator( table = StockJPAUtils.SEQUENCE_TABLE_NAME, name = JPA_SEQUENCE_NAME, pkColumnValue = JPA_COLUMN_NAME, allocationSize = 1 )
    @Id
    @GeneratedValue( strategy = GenerationType.TABLE, generator = JPA_SEQUENCE_NAME )
    @Column( name = "id_category" )
    public int getIdCategory(  )
    {
        return _idCategory;
    }

    /**
     * Set the category id
     * @param idCategory the category id
     */
    public void setIdCategory( int idCategory )
    {
        _idCategory = idCategory;
    }

    /**
     * Return the category status
     * @return the status
     */
    @Column( name = "active" )
    public boolean getActive(  )
    {
        return _status;
    }

    /**
     * Set the category status
     * @param status the status
     */
    public void setActive( boolean status )
    {
        _status = status;
    }

    /**
     * Return the category name
     * @return the name
     */
    @Column( name = "name" )
    public String getName(  )
    {
        return _strName;
    }

    /**
     * Set the category name
     * @param name the category name
     */
    public void setName( String name )
    {
        _strName = name;
    }

    /**
     * Return the children list
     * @return the children list
     */
    @OneToMany( cascade = CascadeType.ALL, fetch = FetchType.LAZY )
    @OrderColumn
    public List<Category> getChildrenList(  )
    {
        return _childrenList;
    }

    /**
     * Set the children list
     * @param childrenList the children list
     */
    public void setChildrenList( List<Category> childrenList )
    {
        _childrenList = childrenList;
    }

    /**
     * Return the product list
     * @return the product list
     */
    @OneToMany( fetch = FetchType.LAZY, cascade = CascadeType.REMOVE )
    @OrderBy( "idProduct DESC" )
    public List<Product> getProductSet(  )
    {
        return _productSet;
    }

    /**
     * Set the product list
     * @param productSet the product list
     */
    public void setProductSet( List<Product> productSet )
    {
        _productSet = productSet;
    }

    /**
     * return the category parent
     * @return the parent
     */
    @ManyToOne( optional = true, fetch = FetchType.LAZY )
    @JoinColumn( name = "parent_id" ) //, nullable = false, updatable = false, insertable = false)

    public Category getParent(  )
    {
        return _parentCategory;
    }

    /**
     * Set the Category parent
     * @param parentCategory the parent
     */
    public void setParent( Category parentCategory )
    {
        this._parentCategory = parentCategory;
    }

    /**
     * Return the provider
     * @return the provider
     */
    @ManyToOne( optional = true, fetch = FetchType.LAZY )
    @JoinColumn( name = "provider_id" )
    public Provider getProvider(  )
    {
        return _provider;
    }

    /**
     * Set the provider
     * @param provider the provider
     */
    public void setProvider( Provider provider )
    {
        _provider = provider;
    }

    /**
     * Return the object definition
     * @return the object definition
     */
    @Transient
    public ObjectDefinition getObjectDefinition(  )
    {
        return GlobalCategoryService.getInstance(  ).getObjectDefinition( this.getClass(  ).getName(  ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode(  )
    {
        final int prime = 31;
        int result = 1;
        result = ( prime * result ) + _idCategory;

        return result;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals( Object obj )
    {
        if ( this == obj )
        {
            return true;
        }

        if ( obj == null )
        {
            return false;
        }

        if ( getClass(  ) != obj.getClass(  ) )
        {
            return false;
        }

        Category other = (Category) obj;

        if ( _idCategory != other._idCategory )
        {
            return false;
        }

        return true;
    }
}
