/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.business;

import fr.paris.lutece.plugins.stock.service.GlobalProductService;
import fr.paris.lutece.plugins.stock.utils.jpa.StockJPAUtils;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.TableGenerator;
import javax.persistence.Transient;


/**
 *
 * Product
 *
 */
@Entity
@Inheritance( strategy = InheritanceType.JOINED )
@Table( name = "stock_product" )
public class Product
{
    /** Sequence name */
    private static final String JPA_SEQUENCE_NAME = "stock_product_sequence";

    /** Unique value */
    private static final String JPA_COLUMN_NAME = "stock_product_id";
    private int _idProduct;
    private float _price;
    private int _stockQuantity;
    private Category _category;

    /**
     * Constructor
     */
    public Product(  )
    {
        super(  );
    }

    /**
     * Constructor from an other product object
     * @param product the product
     */
    public Product( Product product )
    {
        _price = product.getPrice(  );
        _stockQuantity = product.getStockQuantity(  );
        _category = product.getCategory(  );
        _idProduct = product.getIdProduct(  );
    }

    /**
     * Return the product id
     * @return the product id
     */
    @TableGenerator( table = StockJPAUtils.SEQUENCE_TABLE_NAME, name = JPA_SEQUENCE_NAME, pkColumnValue = JPA_COLUMN_NAME, allocationSize = 1 )
    @Id
    @GeneratedValue( strategy = GenerationType.TABLE, generator = JPA_SEQUENCE_NAME )
    @Column( name = "id_product" )
    public int getIdProduct(  )
    {
        return _idProduct;
    }

    /**
     * Set the product id
     * @param idProduct the product id
     */
    public void setIdProduct( int idProduct )
    {
        _idProduct = idProduct;
    }

    /**
     * Return the price
     * @return the price
     */
    @Column( name = "price" )
    public float getPrice(  )
    {
        return _price;
    }

    /**
     * Set the price
     * @param price the price
     */
    public void setPrice( float price )
    {
        _price = price;
    }

    /**
     * Return the quantity of available product
     * @return the stock available
     */
    @Column( name = "stock_quantity" )
    public int getStockQuantity(  )
    {
        return _stockQuantity;
    }

    /**
     * Set the quantity in stock
     * @param stockQuantity the quantity
     */
    public void setStockQuantity( int stockQuantity )
    {
        _stockQuantity = stockQuantity;
    }

    /**
     * Get the category linked to the product
     * @return the category
     */
    @ManyToOne( fetch = FetchType.LAZY )
    public Category getCategory(  )
    {
        return _category;
    }

    /**
     * Set the category linked to the product
     * @param category the category
     */
    public void setCategory( Category category )
    {
        _category = category;
    }

    /**
     * Get the object definition for this kind of product
     * @return the object definition (contains jsp etc..)
     */
    @Transient
    public ObjectDefinition getObjectDefinition(  )
    {
        return GlobalProductService.getInstance(  ).getObjectDefinition( this.getClass(  ).getName(  ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode(  )
    {
        final int prime = 31;
        int result = 1;
        result = ( prime * result ) + _idProduct;

        return result;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals( Object obj )
    {
        if ( this == obj )
        {
            return true;
        }

        if ( obj == null )
        {
            return false;
        }

        if ( getClass(  ) != obj.getClass(  ) )
        {
            return false;
        }

        Product other = (Product) obj;

        if ( _idProduct != other._idProduct )
        {
            return false;
        }

        return true;
    }
}
