/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.service;

import fr.paris.lutece.plugins.stock.business.ObjectDefinition;
import fr.paris.lutece.plugins.stock.business.Product;
import fr.paris.lutece.plugins.stock.business.ProductFilter;
import fr.paris.lutece.portal.business.user.AdminUser;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 * This class provides generics methods for {@link IProductService} implementations
 */
public final class GlobalProductService
{
    private static GlobalProductService _singleton;
    private Map<String, IProductService> _services;

    /**
     * Private constructor
     */
    private GlobalProductService(  )
    {
        _services = new HashMap<String, IProductService>(  );
    }

    /**
     * Get the instance of this class
     * @return the instance of this class
     */
    public static GlobalProductService getInstance(  )
    {
        if ( _singleton == null )
        {
            _singleton = new GlobalProductService(  );
        }

        return _singleton;
    }

    /**
     * Register a {@link IProductService} service
     * @param service the service to be registered
     */
    public void register( IProductService service )
    {
        _services.put( service.getProductClass(  ), service );
    }

    /**
     * Get the creation template of a service
     * @param strProductClassName The class name of the product entity to create
     * @return the relative path of the creation template
     */
    public String getCreateTemplate( String strProductClassName )
    {
        IProductService service = _services.get( strProductClassName );

        return service.getCreateTemplate(  );
    }

    /**
     * Get the modification template of a service
     * @param strProductClassName The class name of the product entity to modify
     * @return the relative path of the modification template
     */
    public String getModifyTemplate( String strProductClassName )
    {
        IProductService service = _services.get( strProductClassName );

        return service.getModifyTemplate(  );
    }

    /**
     * Get the product order template of a service
     * @param strProductClassName the class name of the service
     * @return the relative path of the product order template
     */
    public String getOrderTemplate( String strProductClassName )
    {
        IProductService service = _services.get( strProductClassName );

        return service.getOrderTemplate(  );
    }

    /**
     * Get the deletion jsp of a service
     * @param strProductClassName the class name of the service
     * @return the relative path of the deletion jsp
     */
    public String getDeleteJsp( String strProductClassName )
    {
        IProductService service = _services.get( strProductClassName );

        return service.getDeleteJsp(  );
    }

    /**
     * Creates a product
     * @param strProductClassName The class name of the product entity to create
     * @param product The product to create in database
     * @param request The HTTP request
     * @return error if occurs
     */
    public String doCreateProduct( String strProductClassName, Product product, HttpServletRequest request )
    {
        IProductService service = _services.get( strProductClassName );

        return service.doCreateProduct( product, request );
    }

    /**
     * Modifies a product
     * @param strProductClassName The class name of the product entity to modify
     * @param product The product to modify in database
     * @param request The HTTP request
     * @return error if occurs
     */
    public String doModifyProduct( String strProductClassName, Product product, HttpServletRequest request )
    {
        IProductService service = _services.get( strProductClassName );

        return service.doModifyProduct( product, request );
    }

    /**
     * Deletes a product
     * @param strProductClassName The class name of the product entity to delete
     * @param nIdProduct The product id to delete
     */
    public void doDeleteProduct( String strProductClassName, int nIdProduct )
    {
        IProductService service = _services.get( strProductClassName );
        service.doDeleteProduct( nIdProduct );
    }

    /**
     * Initaliaze the model in parameter for a modification
     * @param strProductClassName The class name of the product entity to delete
     * @param nIdProduct The product id to modify
     * @param model the model which will be passed in the modification template
     * @param request The HTTP request
     */
    public void initModelForModification( String strProductClassName, int nIdProduct, Map<String, Object> model,
        HttpServletRequest request )
    {
        IProductService service = _services.get( strProductClassName );
        service.initModelForModification( nIdProduct, model, request );
    }

    /**
     * Initaliaze the model in parameter for an order
     * @param strProductClassName The class name of the product entity to delete
     * @param nIdProduct The product id to order
     * @param model the model which will be passed in the order template
     * @param request The HTTP request
     */
    public void initModelForOrder( String strProductClassName, int nIdProduct, Map<String, Object> model,
        HttpServletRequest request )
    {
        IProductService service = _services.get( strProductClassName );
        service.initModelForOrder( nIdProduct, model, request );
    }

    /**
     * Get the product with the given id
     * @param strProductClassName The class name of the product entity to delete
     * @param nIdProduct The product id
     * @return The product with the given id
     */
    public Product getProduct( String strProductClassName, int nIdProduct )
    {
        IProductService service = _services.get( strProductClassName );

        return service.getProduct( nIdProduct );
    }

    /**
     * Get all the products which match with the given filter
     * @param strProductClassName The class name of the product entity to delete
     * @param filter the filter
     * @return A list of products which match with the given filter
     */
    public List<Product> findByFilter( String strProductClassName, ProductFilter filter )
    {
        IProductService service = _services.get( strProductClassName );

        return service.findByFilter( filter );
    }

    /**
     * Get the {@link ObjectDefinition} objects of the registered services where the given user is authorized to create
     * @param user the user
     * @return The list of {@link ObjectDefinition} for which the given user is authorized to create a product
     */
    public List<ObjectDefinition> getAllRegisteredObjectDefinition( AdminUser user )
    {
        Collection<IProductService> listServices = _services.values(  );
        List<ObjectDefinition> listAllObjectDefinitions = new ArrayList<ObjectDefinition>(  );

        for ( IProductService service : listServices )
        {
            if ( service.isAuthorizedToCreate( user ) )
            {
                listAllObjectDefinitions.add( service.getObjectDefinition(  ) );
            }
        }

        return listAllObjectDefinitions;
    }

    /**
     * Get the {@link ObjectDefinition} objects of the given product entity name
     * @param strProductClassName The class name of the product entity
     * @return the {@link ObjectDefinition} objects of the given class name
     */
    public ObjectDefinition getObjectDefinition( String strProductClassName )
    {
        IProductService service = _services.get( strProductClassName );

        return service.getObjectDefinition(  );
    }

    /**
     * Checks if a user is authorized to create the given product entity name
     * @param user the user
     * @param strProductClassName The class name of the product entity
     * @return true if the given user is authorized to create the given product entity. False otherwise.
     */
    public boolean isAuthorizedToCreate( AdminUser user, String strProductClassName )
    {
        IProductService service = _services.get( strProductClassName );

        return service.isAuthorizedToCreate( user );
    }

    /**
     * Checks if a user is authorized to modify the given product entity name
     * @param user the user
     * @param strProductClassName The class name of the product entity
     * @return true if the given user is authorized to modify the given product entity. False otherwise.
     */
    public boolean isAuthorizedToModify( AdminUser user, String strProductClassName )
    {
        IProductService service = _services.get( strProductClassName );

        return service.isAuthorizedToModify( user );
    }

    /**
     * Checks if a user is authorized to delete the given product entity name
     * @param user the user
     * @param strProductClassName The class name of the product entity
     * @return true if the given user is authorized to delete the given product entity. False otherwise.
     */
    public boolean isAuthorizedToDelete( AdminUser user, String strProductClassName )
    {
        IProductService service = _services.get( strProductClassName );

        return service.isAuthorizedToDelete( user );
    }

    /**
     * Get the message when a user is not authorized to create a product
     * @param strProductClassName The class name of the product entity
     * @return The message
     */
    public String getMessageNotAuthorizedForCreate( String strProductClassName )
    {
        IProductService service = _services.get( strProductClassName );

        return service.getMessageNotAuthorizedForCreate(  );
    }

    /**
     * Get the message when a user is not authorized to modify a product
     * @param strProductClassName The class name of the product entity
     * @return The message
     */
    public String getMessageNotAuthorizedForModification( String strProductClassName )
    {
        IProductService service = _services.get( strProductClassName );

        return service.getMessageNotAuthorizedForModification(  );
    }

    /**
     * Get the message when a user is not authorized to delete a product
     * @param strProductClassName The class name of the product entity
     * @return The message
     */
    public String getMessageNotAuthorizedForDelete( String strProductClassName )
    {
        IProductService service = _services.get( strProductClassName );

        return service.getMessageNotAuthorizedForDelete(  );
    }
}
