/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.service;

import fr.paris.lutece.plugins.stock.business.Category;
import fr.paris.lutece.plugins.stock.business.CategoryFilter;
import fr.paris.lutece.plugins.stock.business.ObjectDefinition;
import fr.paris.lutece.portal.business.user.AdminUser;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * ICategoryService
 *
 */
public interface ICategoryService
{
    /**
     * Get the creation template of a service
     * @return the relative path of the creation template
     */
    String getCreateTemplate(  );

    /**
     * Get the creation template of a service
     * @return the relative path of the creation template
     */
    String getDuplicateTemplate( );

    /**
     * Get the modification template of a service
     * @return the relative path of the modification template
     */
    String getModifyTemplate(  );

    /**
     * Return the jsp for deleting a category
     * @return the jsp path
     */
    String getDeleteJsp(  );

    /**
    * Get the class of the category object
    * @return the class of the category object
    */
    String getCategoryClass(  );

    /**
     * Init the object
     */
    void init(  );

    /**
    * Creates a category
    * @param category The category to create in database
    * @param request The HTTP request
    * @return error if occurs
    */
    String doCreateCategory( Category category, HttpServletRequest request );

    /**
    * Modifies a category
    * @param category The category to modify in database
    * @param request The HTTP request
    * @return error if occurs
    */
    String doModifyCategory( Category category, HttpServletRequest request );

    /**
    * Deletes a category
    * @param nIdCategory The category id to delete in database
    */
    void doDeleteCategory( int nIdCategory );

    /**
    * Initaliaze the model in parameter for a modification
    * @param nIdCategory The category id to modify
    * @param model the model which will be passed in the modification template
    */
    void initModelForModification( int nIdCategory, Map<String, Object> model );

    /**
    * Initaliaze the model in parameter for a creation
    * @param model the model which will be passed in the modification template
    */
    void initModelForCreation( Map<String, Object> model );

    /**
     * Initaliaze the model in parameter for a duplication
     * @param model the model which will be passed in the creation template
     */
    void initModelForDuplication( int nIdCategory, Map<String, Object> model );

    /**
    * Get the category with the given id
    * @param nIdCategory The category id
    * @return The category with the given id
    */
    Category getCategory( int nIdCategory );

    /**
     * Return the ObjectDefinition object of the service
     * @return The ObjectDefintion object
     */
    ObjectDefinition getObjectDefinition(  );

    /**
     * Finds a category and fetch parent
     * @param nIdCategory the id of the searched category
     * @return the category
     */
    Category getCategoryWithParent( int nIdCategory );

    /**
     * Finds a category and all his subcategories and fetch children
     * @param nIdCategory the id of upper category
     * @return the category
     */
    Category getCategoryWithChildren( Integer nIdCategory );

    /**
    * Finds all categories which match the given filter
    * @param filter The filter
    * @return A list of categories which match the given filter
    */
    List<Category> findByFilter( CategoryFilter filter );

    /**
    * Checks if the given user is authorized to create a category
    * @param user the user
    * @return true if the given user is authorized to create a category. False otherwise.
    */
    boolean isAuthorizedToCreate( AdminUser user );

    /**
    * Checks if the given user is authorized to modify a category
    * @param user the user
    * @return true if the given user is authorized to modify a category. False otherwise.
    */
    boolean isAuthorizedToModify( AdminUser user );

    /**
    * Checks if the given user is authorized to delete a category
    * @param user the user
    * @return true if the given user is authorized to delete a category. False otherwise.
    */
    boolean isAuthorizedToDelete( AdminUser user );

    /**
    * Get the message when a user is not authorized to create a category
    * @return The message
    */
    String getMessageNotAuthorizedForCreate(  );

    /**
    * Get the message when a user is not authorized to modify a category
    * @return The message
    */
    String getMessageNotAuthorizedForModification(  );

    /**
    * Get the message when a user is not authorized to delete a category
    * @return The message
    */
    String getMessageNotAuthorizedForDelete(  );
}
