/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.service;

import fr.paris.lutece.plugins.stock.business.Category;
import fr.paris.lutece.plugins.stock.business.ICategoryHome;
import fr.paris.lutece.plugins.stock.business.IProductHome;
import fr.paris.lutece.plugins.stock.business.ObjectDefinition;
import fr.paris.lutece.plugins.stock.business.Product;
import fr.paris.lutece.plugins.stock.business.ProductFilter;
import fr.paris.lutece.plugins.stock.utils.constants.StockConstants;
import fr.paris.lutece.plugins.stock.web.StockJspBean;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.business.user.AdminUser;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.rbac.RBACService;

import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * ProductService
 *
 */
@Transactional( readOnly = true )
public final class ProductService implements IProductService
{
    //TEMPLATES
    private static final String TEMPLATE_CREATE_PRODUCT = "/admin/plugins/stock/create_product.html";
    private static final String TEMPLATE_MODIFY_PRODUCT = "/admin/plugins/stock/modify_product.html";
    private static final String TEMPLATE_ORDER_PRODUCT = "/admin/plugins/stock/order_product.html";

    //MARKS
    private static final String MARK_PRODUCT = "product";
    private static final String PRODUCT_NAME_KEY = "stock.create_product.type.basic.name";

    //MESSAGE
    private static final String MESSAGE_NOT_AUTHORIZED_TO_CREATE_BASIC_PRODUCT = "stock.create_product.error.basicproduct.notAuthorized";
    private static final String MESSAGE_NOT_AUTHORIZED_TO_DELETE_BASIC_PRODUCT = "stock.delete_product.error.basicproduct.notAuthorized";
    private static final String MESSAGE_NOT_AUTHORIZED_TO_MODIFY_BASIC_PRODUCT = "stock.modify_product.error.basicproduct.notAuthorized";
    private static final String MESSAGE_ERROR_OCCUR = "stock.message.error.errorOccur";

    //JSP
    private static final String PRODUCT_CREATION_JSP = "jsp/admin/plugins/stock/CreateProduct.jsp";
    private static final String PRODUCT_MODIFICATION_JSP = "jsp/admin/plugins/stock/ModifyProduct.jsp";
    private static final String PRODUCT_DELETION_JSP = "jsp/admin/plugins/stock/DeleteProduct.jsp";
    private static final String JSP_DO_DELETE_PRODUCT = "jsp/admin/plugins/stock/DoDeleteProduct.jsp";
    private static ProductService _singleton;
    private static ObjectDefinition _objectDefinition;
    private IProductHome<Product, ProductFilter> _homeProduct;
    private ICategoryHome _homeCategory;

    /**
     * Constructor
     */
    private ProductService(  )
    {
    }

    /**
     * {@inheritDoc}
     *
     */
    public IProductHome<Product, ProductFilter> getHomeProduct(  )
    {
        return _homeProduct;
    }

    /**
     * {@inheritDoc}
     *
     */
    public void setHomeProduct( IProductHome<Product, ProductFilter> homeProduct )
    {
        _homeProduct = homeProduct;
    }

    /**
     * {@inheritDoc}
     *
     */
    public ICategoryHome getHomeCategory(  )
    {
        return _homeCategory;
    }

    /**
     * {@inheritDoc}
     *
     */
    public void setHomeCategory( ICategoryHome homeCategory )
    {
        _homeCategory = homeCategory;
    }

    /**
     *
     *{@inheritDoc}
     */
    public String getProductClass(  )
    {
        return Product.class.getName(  );
    }

    /**
     *
     *{@inheritDoc}
     */
    public String getCreateTemplate(  )
    {
        return TEMPLATE_CREATE_PRODUCT;
    }

    /**
     *
     *{@inheritDoc}
     */
    public String getModifyTemplate(  )
    {
        return TEMPLATE_MODIFY_PRODUCT;
    }

    /**
     *
     *{@inheritDoc}
     */
    public String getOrderTemplate(  )
    {
        return TEMPLATE_ORDER_PRODUCT;
    }

    /**
     *
     *{@inheritDoc}
     */
    public void init(  )
    {
    }

    /**
     *
     *{@inheritDoc}
     */
    public static ProductService getInstance(  )
    {
        if ( _singleton == null )
        {
            _singleton = new ProductService(  );
        }

        return _singleton;
    }

    /**
     *
     *{@inheritDoc}
     */
    @Transactional
    public String doCreateProduct( Product product, HttpServletRequest request )
    {
        Category productCategory = _homeCategory.findByIdWithProduct( product.getCategory(  ).getIdCategory(  ) );

        if ( productCategory != null )
        {
            productCategory.getProductSet(  ).add( product );
            _homeProduct.create( product );
        }
        else
        {
            return MESSAGE_ERROR_OCCUR;
        }

        return null;
    }

    /**
     *
     *{@inheritDoc}
     */
    @Transactional
    public String doModifyProduct( Product product, HttpServletRequest request )
    {
        try
        {
            product.setPrice( Float.parseFloat( request.getParameter( StockConstants.PARAMETER_PRODUCT_PRICE ) ) );
            product.setStockQuantity( Integer.parseInt( request.getParameter( StockConstants.PARAMETER_PRODUCT_QUANTITY ) ) );
        }
        catch ( NumberFormatException e )
        {
            return AdminMessageService.getMessageUrl( request, StockConstants.MESSAGE_ERROR_NAN, AdminMessage.TYPE_STOP );
        }

        _homeProduct.update( product );

        return null;
    }

    /**
     *
     *{@inheritDoc}
     */
    @Transactional
    public void doDeleteProduct( int nIdProduct )
    {
        Product product = _homeProduct.findByPrimaryKey( nIdProduct );
        Category category = _homeCategory.findByIdWithProduct( product.getCategory(  ).getIdCategory(  ) );
        category.getProductSet(  ).remove( product );
        _homeCategory.update( category );
        _homeProduct.remove( nIdProduct );
    }

    /**
     *
     *{@inheritDoc}
     */
    public void initModelForModification( int nIdProduct, Map<String, Object> model, HttpServletRequest request )
    {
        Product product = _homeProduct.findByPrimaryKey( nIdProduct );
        model.put( MARK_PRODUCT, product );
    }

    /**
     *
     *{@inheritDoc}
     */
    public void initModelForOrder( int nIdProduct, Map<String, Object> model, HttpServletRequest request )
    {
        Product product = _homeProduct.findByPrimaryKey( nIdProduct );
        model.put( MARK_PRODUCT, product );
    }

    /**
     *
     *{@inheritDoc}
     */
    public Product getProduct( int nIdProduct )
    {
        return _homeProduct.findByPrimaryKey( nIdProduct );
    }

    /**
     *
     *{@inheritDoc}
     */
    public String getDeleteJsp(  )
    {
        return JSP_DO_DELETE_PRODUCT;
    }

    /**
     *
     *{@inheritDoc}
     */
    public List<Product> findByFilter( ProductFilter filter )
    {
        return _homeProduct.findByFilter( filter );
    }

    /**
     *
     *{@inheritDoc}
     */
    public ObjectDefinition getObjectDefinition(  )
    {
        if ( _objectDefinition == null )
        {
            _objectDefinition = new ObjectDefinition( PRODUCT_NAME_KEY, PRODUCT_CREATION_JSP, PRODUCT_MODIFICATION_JSP,
                    PRODUCT_DELETION_JSP );
        }

        return _objectDefinition;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAuthorizedToCreate( AdminUser user )
    {
        if ( !RBACService.isAuthorized( StockJspBean.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    StockResourceIdService.PERMISSION_CREATE, user ) )
        {
            return false;
        }

        return true;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAuthorizedToModify( AdminUser user )
    {
        if ( !RBACService.isAuthorized( StockJspBean.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    StockResourceIdService.PERMISSION_MODIFY, user ) )
        {
            return false;
        }

        return true;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAuthorizedToDelete( AdminUser user )
    {
        if ( !RBACService.isAuthorized( StockJspBean.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    StockResourceIdService.PERMISSION_DELETE, user ) )
        {
            return false;
        }

        return true;
    }

    /**
     * {@inheritDoc}
     *
     */
    public String getMessageNotAuthorizedForCreate(  )
    {
        return MESSAGE_NOT_AUTHORIZED_TO_CREATE_BASIC_PRODUCT;
    }

    /**
     * {@inheritDoc}
     *
     */
    public String getMessageNotAuthorizedForDelete(  )
    {
        return MESSAGE_NOT_AUTHORIZED_TO_DELETE_BASIC_PRODUCT;
    }

    /**
     * {@inheritDoc}
     *
     */
    public String getMessageNotAuthorizedForModification(  )
    {
        return MESSAGE_NOT_AUTHORIZED_TO_MODIFY_BASIC_PRODUCT;
    }

    /**
     * {@inheritDoc}
     *
     */
    public void updateProduct( Product product )
    {
        _homeProduct.update( product );
    }
}
