/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.service;

import fr.paris.lutece.plugins.stock.business.IProviderHome;
import fr.paris.lutece.plugins.stock.business.ObjectDefinition;
import fr.paris.lutece.plugins.stock.business.Provider;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.business.user.AdminUser;
import fr.paris.lutece.portal.service.rbac.RBACService;

import org.springframework.transaction.annotation.Transactional;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * ProviderService
 *
 */
@Transactional( readOnly = true )
public final class ProviderService implements IProviderService
{
    private static final String TEMPLATE_CREATE_PROVIDER = "/admin/plugins/stock/create_provider.html";
    private static final String TEMPLATE_MODIFY_PROVIDER = "/admin/plugins/stock/modify_provider.html";
    private static final String MARK_PROVIDER = "provider";
    private static final String PROVIDER_NAME_KEY = "stock.create_provider.type.basic.name";

    //JSP
    private static final String PROVIDER_CREATION_JSP = "jsp/admin/plugins/stock/CreateProvider.jsp";
    private static final String PROVIDER_MODIFICATION_JSP = "jsp/admin/plugins/stock/ModifyProvider.jsp";
    private static final String PROVIDER_DELETION_JSP = "jsp/admin/plugins/stock/DeleteProvider.jsp";
    private static final String PROVIDER_DO_DELETE_JSP = "jsp/admin/plugins/stock/DoDeleteProvider.jsp";
    private static ProviderService _instance = new ProviderService(  );
    private static ObjectDefinition _objectDefinition;
    private IProviderHome _homeProvider;

    /**
     * Constructor
     */
    private ProviderService(  )
    {
    }

    /**
     * Return the home provider
     * @return the home provider
     */
    public IProviderHome getHomeProvider(  )
    {
        return _homeProvider;
    }

    /**
     * Set the home provider
     * @param homeProvider the home provider
     */
    public void setHomeProvider( IProviderHome homeProvider )
    {
        _homeProvider = homeProvider;
    }

    /**
     * Return the provider service instance
     * @return the provider service instance
     */
    public static ProviderService getInstance(  )
    {
        return _instance;
    }

    /**
     * {@inheritDoc}
     */
    public String getCreateTemplate(  )
    {
        return TEMPLATE_CREATE_PROVIDER;
    }

    /**
     * {@inheritDoc}
     */
    public String getModifyTemplate(  )
    {
        return TEMPLATE_MODIFY_PROVIDER;
    }

    /**
     * {@inheritDoc}
     */
    public String getDeleteJsp(  )
    {
        return PROVIDER_DO_DELETE_JSP;
    }

    /**
     * {@inheritDoc}
     */
    public String getProviderClass(  )
    {
        return Provider.class.getName(  );
    }

    /**
     * {@inheritDoc}
     */
    public void init(  )
    {
    }

    /**
     * {@inheritDoc}
     */
    @Transactional
    public void doCreateProvider( Provider provider, HttpServletRequest request )
    {
        _homeProvider.create( provider );
    }

    /**
     * {@inheritDoc}
     */
    @Transactional
    public void doModifyProvider( Provider provider, HttpServletRequest request )
    {
        _homeProvider.update( provider );
    }

    /**
     * {@inheritDoc}
     */
    @Transactional
    public void doDeleteProvider( int nIdProvider )
    {
        _homeProvider.remove( nIdProvider );
    }

    /**
     * {@inheritDoc}
     */
    public void initModelForCreation( Map<String, Object> model )
    {
        // Nothing to do
    }

    /**
     * {@inheritDoc}
     */
    public void initModelForModification( int nIdProvider, Map<String, Object> model )
    {
        Provider provider = _homeProvider.findByPrimaryKey( nIdProvider );
        model.put( MARK_PROVIDER, provider );
    }

    /**
     * {@inheritDoc}
     */
    public ObjectDefinition getObjectDefinition(  )
    {
        if ( _objectDefinition == null )
        {
            _objectDefinition = new ObjectDefinition( PROVIDER_NAME_KEY, PROVIDER_CREATION_JSP,
                    PROVIDER_MODIFICATION_JSP, PROVIDER_DELETION_JSP );
        }

        return _objectDefinition;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAuthorizedToCreate( AdminUser user )
    {
        if ( !RBACService.isAuthorized( Provider.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    ProviderIdService.PERMISSION_CREATE, user ) )
        {
            return false;
        }

        return true;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAuthorizedToModify( AdminUser user )
    {
        if ( !RBACService.isAuthorized( Provider.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    ProviderIdService.PERMISSION_MODIFY, user ) )
        {
            return false;
        }

        return true;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAuthorizedToDelete( AdminUser user )
    {
        if ( !RBACService.isAuthorized( Provider.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    ProviderIdService.PERMISSION_DELETE, user ) )
        {
            return false;
        }

        return true;
    }
}
