/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.service;

import fr.paris.lutece.plugins.stock.business.Category;
import fr.paris.lutece.plugins.stock.business.CategoryFilter;
import fr.paris.lutece.plugins.stock.business.IProviderHome;
import fr.paris.lutece.plugins.stock.business.ITicketCategoryHome;
import fr.paris.lutece.plugins.stock.business.ObjectDefinition;
import fr.paris.lutece.plugins.stock.business.Provider;
import fr.paris.lutece.plugins.stock.business.TicketCategory;
import fr.paris.lutece.plugins.stock.utils.StockUtils;
import fr.paris.lutece.plugins.stock.utils.constants.StockConstants;
import fr.paris.lutece.plugins.stock.web.StockJspBean;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.business.user.AdminUser;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.util.AppLogService;

import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * TicketCategoryService
 *
 */
@Transactional( readOnly = true )
public final class TicketCategoryService implements ICategoryService
{
    private static final String TEMPLATE_CREATE_TICKET_CATEGORY = "/admin/plugins/stock/create_ticket_category.html";
    private static final String TEMPLATE_MODIFY_TICKET_CATEGORY = "/admin/plugins/stock/modify_ticket_category.html";
    private static final String TICKET_CATEGORY_NAME_KEY = "stock.create_category.type.ticket.name";

    //JSP
    private static final String TICKET_CATEGORY_CREATION_JSP = "jsp/admin/plugins/stock/CreateTicketCategory.jsp";
    private static final String TEMPLATE_DUPLICATE_CATEGORY = "/admin/plugins/stock/duplicate_category.html";
    private static final String TICKET_CATEGORY_MODIFICATION_JSP = "jsp/admin/plugins/stock/ModifyTicketCategory.jsp";
    private static final String TICKET_CATEGORY_DELETION_JSP = "jsp/admin/plugins/stock/DeleteTicketCategory.jsp";
    private static final String TICKET_CATEGORY_DO_DELETE_JSP = "jsp/admin/plugins/stock/DoDeleteTicketCategory.jsp";

    // PARAMETERS
    private static final String PARAMETER_BOOKING_PHONE = "category_booking_phone";
    private static final String PARAMETER_BOOKING_PHONE2 = "category_booking_phone2";
    private static final String PARAMETER_BOOKING_MAIL = "category_booking_mail";

    //MESSAGE
    private static final String MESSAGE_NOT_AUTHORIZED_TO_CREATE_TICKET_CATEGORY = "stock.create_category.error.ticketcategory.notAuthorized";
    private static final String MESSAGE_NOT_AUTHORIZED_TO_DELETE_TICKET_CATEGORY = "stock.delete_category.error.ticketcategory.notAuthorized";
    private static final String MESSAGE_NOT_AUTHORIZED_TO_MODIFY_TICKET_CATEGORY = "stock.modify_category.error.ticketcategory.notAuthorized";
    private static TicketCategoryService _instance = new TicketCategoryService(  );
    private static ObjectDefinition _objectDefinition;
    private ITicketCategoryHome<TicketCategory> _homeTicketCategory;
    private IProviderHome _homeProvider;

    /**
     * Private constructor
     */
    private TicketCategoryService(  )
    {
        //_homeTicketCategory = ( ITicketCategoryHome )SpringContextService.getPluginBean( StockPlugin.PLUGIN_NAME, "stock.ticketCategoryHome" );
    }

    /**
     * Get the provider home
     * @return the provider home
     */
    public IProviderHome getHomeProvider(  )
    {
        return _homeProvider;
    }

    /**
     * Set the provider home
     * @param homeProvider the new provider home
     */
    public void setHomeProvider( IProviderHome homeProvider )
    {
        _homeProvider = homeProvider;
    }

    /**
     * Get the {@link ITicketCategoryHome} object
     * @return the {@link ITicketCategoryHome} object
     */
    public ITicketCategoryHome<TicketCategory> getHomeTicketCategory(  )
    {
        return _homeTicketCategory;
    }

    /**
     * Set the {@link ITicketCategoryHome} object
     * @param homeTicketCategory the new {@link ITicketCategoryHome} object
     */
    public void setHomeTicketCategory( ITicketCategoryHome<TicketCategory> homeTicketCategory )
    {
        _homeTicketCategory = homeTicketCategory;
    }

    /**
     * Get the single instance of this class
     * @return the single instance of this class
     */
    public static TicketCategoryService getInstance(  )
    {
        return _instance;
    }

    /**
     * {@inheritDoc}
     */
    public String getCreateTemplate(  )
    {
        return TEMPLATE_CREATE_TICKET_CATEGORY;
    }

    /**
     * {@inheritDoc}
     */
    public String getDuplicateTemplate( )
    {
        return TEMPLATE_DUPLICATE_CATEGORY;
    }

    /**
     * {@inheritDoc}
     */
    public String getModifyTemplate(  )
    {
        return TEMPLATE_MODIFY_TICKET_CATEGORY;
    }

    /**
     * Return the jsp for deleting a product
     * @return the jsp path
     */
    public String getDeleteJsp(  )
    {
        return TICKET_CATEGORY_DO_DELETE_JSP;
    }

    /**
     * {@inheritDoc}
     */
    public String getCategoryClass(  )
    {
        return TicketCategory.class.getName(  );
    }

    /**
     * {@inheritDoc}
     */
    public void init(  )
    {
    }

    /**
     * {@inheritDoc}
     */
    @Transactional
    public String doCreateCategory( Category category, HttpServletRequest request )
    {
        TicketCategory ticketCategory = buildTicketCategory( category, request );
        Category parent = category.getParent(  );

        if ( parent != null )
        {
            Category trueParent = _homeTicketCategory.findByPrimaryKey( parent.getIdCategory(  ) );

            if ( trueParent != null )
            {
                trueParent.getChildrenList(  ).add( ticketCategory );
            }
            else
            {
                return StockUtils.MESSAGE_ERROR_OCCUR;
            }
        }

        _homeTicketCategory.create( ticketCategory );
        //to get the id category in jspbean
        category.setIdCategory( ticketCategory.getIdCategory(  ) );

        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Transactional
    public String doModifyCategory( Category category, HttpServletRequest request )
    {
        TicketCategory ticketCategory = buildTicketCategory( category, request );
        _homeTicketCategory.update( ticketCategory );

        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Transactional
    public void doDeleteCategory( int nIdCategory )
    {
        Category categoryToDelete = _homeTicketCategory.findByPrimaryKey( nIdCategory );

        // Remove the ticket category from the provider's categories
        Provider provider = _homeProvider.findByPrimaryKey( categoryToDelete.getProvider(  ).getIdProvider(  ) );
        provider.getProducts(  ).remove( categoryToDelete );
        _homeProvider.update( provider );
        // Remove the ticket category
        _homeTicketCategory.remove( nIdCategory );
    }

    /**
     * {@inheritDoc}
     */
    public void initModelForModification( int nIdCategory, Map<String, Object> model )
    {
        Category category = getCategory( nIdCategory );
        model.put( StockConstants.MARK_CATEGORY, category );
    }

    /**
     * {@inheritDoc}
     */
    public void initModelForCreation( Map<String, Object> model )
    {
        //Nothing to do
    }

    /**
     * {@inheritDoc}
     */
    public void initModelForDuplication( int nIdCategory, Map<String, Object> model )
    {
        //Nothing to do
    }

    /**
     * {@inheritDoc}
     */
    public Category getCategory( int nIdCategory )
    {
        return _homeTicketCategory.findByPrimaryKey( nIdCategory );
    }

    /**
     *
     *{@inheritDoc}
     */
    public Category getCategoryWithParent( int nIdCategory )
    {
        return _homeTicketCategory.findByIdWithParent( nIdCategory );
    }

    /**
     * {@inheritDoc}
     */
    public ObjectDefinition getObjectDefinition(  )
    {
        if ( _objectDefinition == null )
        {
            _objectDefinition = new ObjectDefinition( TICKET_CATEGORY_NAME_KEY, TICKET_CATEGORY_CREATION_JSP,
                    TICKET_CATEGORY_MODIFICATION_JSP, TICKET_CATEGORY_DELETION_JSP );
        }

        return _objectDefinition;
    }

    /**
     * Builds a {@link TicketCategory} object with the parameter of the given request and category object
     * @param category the initial category
     * @param request the Http request
     * @return the builded {@link TicketCategory} object
     */
    private TicketCategory buildTicketCategory( Category category, HttpServletRequest request )
    {
        TicketCategory ticketCategory = new TicketCategory( category );

        String strBookingPhone = request.getParameter( PARAMETER_BOOKING_PHONE );
        String strBookingPhone2 = request.getParameter( PARAMETER_BOOKING_PHONE2 );
        String strBookingMail = request.getParameter( PARAMETER_BOOKING_MAIL );

        try
        {
            if ( strBookingPhone != null )
            {
                Long.valueOf( strBookingPhone );
                ticketCategory.setBookingPhone( strBookingPhone );
            }
        }
        catch ( NumberFormatException e )
        {
            AppLogService.error( e );

            //FIXME: renvoyer l'erreur ci dessous
            //    		return AdminMessageService.getMessageUrl( request, StockConstants.MESSAGE_ERROR_PHONE_NUMBER_FORMAT, AdminMessage.TYPE_STOP );
        }

        try
        {
            if ( strBookingPhone2 != null )
            {
                Long.valueOf( strBookingPhone2 );
                ticketCategory.setBookingPhone( strBookingPhone2 );
            }
        }
        catch ( NumberFormatException e )
        {
            AppLogService.error( e );

            //FIXME: renvoyer l'erreur ci dessous
            //          return AdminMessageService.getMessageUrl( request, StockConstants.MESSAGE_ERROR_PHONE_NUMBER_FORMAT, AdminMessage.TYPE_STOP );
        }

        ticketCategory.setBookingMail( strBookingMail );

        return ticketCategory;
    }

    /**
     * {@inheritDoc}
     */
    public Category getCategoryWithChildren( Integer idCategory )
    {
        return _homeTicketCategory.findByPrimarykeyWithChildren( idCategory );
    }

    /**
     * {@inheritDoc}
     */
    public List<Category> findByFilter( CategoryFilter filter )
    {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAuthorizedToCreate( AdminUser user )
    {
        if ( !RBACService.isAuthorized( StockJspBean.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    StockResourceIdService.PERMISSION_CREATE_CATEGORY, user ) )
        {
            return false;
        }

        return true;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAuthorizedToModify( AdminUser user )
    {
        if ( !RBACService.isAuthorized( StockJspBean.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    StockResourceIdService.PERMISSION_MODIFY_CATEGORY, user ) )
        {
            return false;
        }

        return true;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAuthorizedToDelete( AdminUser user )
    {
        if ( !RBACService.isAuthorized( StockJspBean.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    StockResourceIdService.PERMISSION_DELETE_CATEGORY, user ) )
        {
            return false;
        }

        return true;
    }

    /**
     * {@inheritDoc}
     */
    public String getMessageNotAuthorizedForCreate(  )
    {
        return MESSAGE_NOT_AUTHORIZED_TO_CREATE_TICKET_CATEGORY;
    }

    /**
     * {@inheritDoc}
     */
    public String getMessageNotAuthorizedForDelete(  )
    {
        return MESSAGE_NOT_AUTHORIZED_TO_DELETE_TICKET_CATEGORY;
    }

    /**
     * {@inheritDoc}
     */
    public String getMessageNotAuthorizedForModification(  )
    {
        return MESSAGE_NOT_AUTHORIZED_TO_MODIFY_TICKET_CATEGORY;
    }
}
