/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.service;

import fr.paris.lutece.plugins.stock.business.Category;
import fr.paris.lutece.plugins.stock.business.ICategoryHome;
import fr.paris.lutece.plugins.stock.business.ITicketProductHome;
import fr.paris.lutece.plugins.stock.business.ObjectDefinition;
import fr.paris.lutece.plugins.stock.business.Product;
import fr.paris.lutece.plugins.stock.business.ProductFilter;
import fr.paris.lutece.plugins.stock.business.TicketProduct;
import fr.paris.lutece.plugins.stock.business.TicketProductFilter;
import fr.paris.lutece.plugins.stock.business.WeekDaysEnum;
import fr.paris.lutece.plugins.stock.utils.StockUtils;
import fr.paris.lutece.plugins.stock.utils.constants.StockConstants;
import fr.paris.lutece.plugins.stock.web.StockJspBean;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.business.user.AdminUser;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.util.date.DateUtil;

import org.springframework.transaction.annotation.Transactional;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * TicketProductService
 *
 */
@Transactional( readOnly = true )
public final class TicketProductService implements IProductService
{
    //templates
    private static final String TEMPLATE_CREATE_PRODUCT = "/admin/plugins/stock/create_ticket_product.html";
    private static final String TEMPLATE_MODIFY_PRODUCT = "/admin/plugins/stock/modify_ticket_product.html";
    private static final String TEMPLATE_ORDER_PRODUCT = "/admin/plugins/stock/order_ticket_product.html";

    //marks
    private static final String MARK_PRODUCT = "product";

    //parameters
    private static final String PARAMETER_DURATION = "product_duration";
    private static final String PARAMETER_HOUR = "product_hour";
    private static final String PARAMETER_DAYS = "product_day";
    private static final String PARAMETER_DATE_BEGIN = "date_begin";
    private static final String PARAMETER_DATE_END = "date_end";
    private static final String CATEGORY_NAME_KEY = "stock.create_product.type.ticket.name";

    //JSP
    private static final String CATEGORY_CREATION_JSP = "jsp/admin/plugins/stock/CreateTicketProduct.jsp";
    private static final String CATEGORY_MODIFICATION_JSP = "jsp/admin/plugins/stock/ModifyTicketProduct.jsp";
    private static final String CATEGORY_DELETION_JSP = "jsp/admin/plugins/stock/DeleteTicketProduct.jsp";
    private static final String JSP_DO_DELETE_TICKET_PRODUCT = "jsp/admin/plugins/stock/DoDeleteTicketProduct.jsp";

    //MESSAGE
    private static final String MESSAGE_NOT_AUTHORIZED_TO_CREATE_TICKET_PRODUCT = "stock.create_product.error.ticketproduct.notAuthorized";
    private static final String MESSAGE_NOT_AUTHORIZED_TO_DELETE_TICKET_PRODUCT = "stock.delete_product.error.ticketproduct.notAuthorized";
    private static final String MESSAGE_NOT_AUTHORIZED_TO_MODIFY_TICKET_PRODUCT = "stock.modify_product.error.ticketproduct.notAuthorized";
    private static final String MESSAGE_ERROR_OCCUR = "stock.message.error.errorOccur";
    private static TicketProductService _singleton;
    private static ObjectDefinition _objectDefinition;
    private ITicketProductHome<TicketProduct, TicketProductFilter> _homeTicketProduct;
    private ICategoryHome _homeCategory;

    /**
     * Private constructor
     */
    private TicketProductService(  )
    {
    }

    /**
     * Get the {@link ITicketProductHome} object
     * @return the {@link ITicketProductHome} object
     */
    public ITicketProductHome<TicketProduct, TicketProductFilter> getHomeTicketProduct(  )
    {
        return _homeTicketProduct;
    }

    /**
     * Set the {@link ITicketProductHome} object
     * @param homeTicketProduct the new {@link ITicketProductHome} object
     */
    public void setHomeTicketProduct( ITicketProductHome<TicketProduct, TicketProductFilter> homeTicketProduct )
    {
        _homeTicketProduct = homeTicketProduct;
    }

    /**
     * Get the {@link ICategoryHome} object
     * @return the {@link ICategoryHome} object
     */
    public ICategoryHome getHomeCategory(  )
    {
        return _homeCategory;
    }

    /**
     * Set the the {@link ICategoryHome} object
     * @param homeCategory the new the {@link ICategoryHome} object
     */
    public void setHomeCategory( ICategoryHome homeCategory )
    {
        _homeCategory = homeCategory;
    }

    /**
     *
     *{@inheritDoc}
     */
    public String getProductClass(  )
    {
        return TicketProduct.class.getName(  );
    }

    /**
     *
     *{@inheritDoc}
     */
    public String getCreateTemplate(  )
    {
        return TEMPLATE_CREATE_PRODUCT;
    }

    /**
     *
     *{@inheritDoc}
     */
    public String getModifyTemplate(  )
    {
        return TEMPLATE_MODIFY_PRODUCT;
    }

    /**
     *
     *{@inheritDoc}
     */
    public String getOrderTemplate(  )
    {
        return TEMPLATE_ORDER_PRODUCT;
    }

    /**
     *
     *{@inheritDoc}
     */
    public String getDeleteJsp(  )
    {
        return JSP_DO_DELETE_TICKET_PRODUCT;
    }

    /**
     *
     *{@inheritDoc}
     */
    public void init(  )
    {
    }

    /**
     *
     *{@inheritDoc}
     */
    public static TicketProductService getInstance(  )
    {
        if ( _singleton == null )
        {
            _singleton = new TicketProductService(  );
        }

        return _singleton;
    }

    /**
     *
     *{@inheritDoc}
     */
    @Transactional
    public String doCreateProduct( Product product, HttpServletRequest request )
    {
        TicketProduct ticketProduct = new TicketProduct( product );

        String strHour = request.getParameter( PARAMETER_HOUR );
        String strDateBegin = request.getParameter( PARAMETER_DATE_BEGIN );
        String strDateEnd = request.getParameter( PARAMETER_DATE_END );
        Timestamp hourTs = null;

        if ( ( strHour != null ) && !strHour.equals( StockUtils.EMPTY_STRING ) )
        {
            hourTs = StockUtils.formatHour( strHour, request.getLocale(  ) );
        }

        if ( strDateBegin != null )
        {
            ticketProduct.setDateBegin( DateUtil.formatTimestamp( strDateBegin, request.getLocale(  ) ) );
        }

        if ( strDateEnd != null )
        {
            ticketProduct.setDateBegin( DateUtil.formatTimestamp( strDateEnd, request.getLocale(  ) ) );
        }

        String[] strDays = (String[]) request.getParameterValues( PARAMETER_DAYS );

        ticketProduct.setHour( hourTs );

        List<WeekDaysEnum> weekDays = new ArrayList<WeekDaysEnum>(  );

        for ( String strDay : strDays )
        {
            try
            {
                weekDays.add( WeekDaysEnum.valueOf( strDay ) );
            }
            catch ( IllegalArgumentException e )
            {
                AppLogService.error( e.getMessage(  ) );
            }
        }

        ticketProduct.setDays( weekDays );

        Category productCategory = _homeCategory.findByIdWithProduct( ticketProduct.getCategory(  ).getIdCategory(  ) );

        if ( productCategory != null )
        {
            productCategory.getProductSet(  ).add( ticketProduct );

            _homeTicketProduct.create( ticketProduct );
        }
        else
        {
            return MESSAGE_ERROR_OCCUR;
        }

        return null;
    }

    /**
     *{@inheritDoc}
     */
    @Transactional
    public String doModifyProduct( Product product, HttpServletRequest request )
    {
        try
        {
            product.setPrice( Float.parseFloat( request.getParameter( StockConstants.PARAMETER_PRODUCT_PRICE ) ) );
            product.setStockQuantity( Integer.parseInt( request.getParameter( StockConstants.PARAMETER_PRODUCT_QUANTITY ) ) );
        }
        catch ( NumberFormatException e )
        {
            return AdminMessageService.getMessageUrl( request, StockConstants.MESSAGE_ERROR_NAN, AdminMessage.TYPE_STOP );
        }

        TicketProduct ticketProduct = new TicketProduct( product );

        String strDuration = request.getParameter( PARAMETER_DURATION );
        String strHour = request.getParameter( PARAMETER_HOUR );
        Timestamp hourTs = null;

        if ( ( strHour != null ) && !strHour.equals( StockUtils.EMPTY_STRING ) )
        {
            hourTs = StockUtils.formatHour( strHour, request.getLocale(  ) );
        }

        String[] strDays = (String[]) request.getParameterMap(  ).get( PARAMETER_DAYS );

        try
        {
            ticketProduct.setDuration( Integer.parseInt( strDuration ) );
            ticketProduct.setHour( hourTs );
        }
        catch ( NumberFormatException e )
        {
            return AdminMessageService.getMessageUrl( request, StockUtils.MESSAGE_ERROR_OCCUR, AdminMessage.TYPE_STOP );
        }

        List<WeekDaysEnum> weekDays = new ArrayList<WeekDaysEnum>(  );

        for ( String strDay : strDays )
        {
            try
            {
                weekDays.add( WeekDaysEnum.valueOf( strDay ) );
            }
            catch ( IllegalArgumentException e )
            {
                AppLogService.error( e.getMessage(  ) );
            }
        }

        ticketProduct.setDays( weekDays );

        _homeTicketProduct.update( ticketProduct );

        return null;
    }

    /**
     *{@inheritDoc}
     */
    @Transactional
    public void doDeleteProduct( int nIdProduct )
    {
        Product product = _homeTicketProduct.findByPrimaryKey( nIdProduct );
        Category category = _homeCategory.findByIdWithProduct( product.getCategory(  ).getIdCategory(  ) );
        category.getProductSet(  ).remove( product );
        _homeCategory.update( category );
        _homeTicketProduct.remove( nIdProduct );
    }

    /**
     *{@inheritDoc}
     */
    public void initModelForModification( int nIdProduct, Map<String, Object> model, HttpServletRequest request )
    {
        TicketProduct product = _homeTicketProduct.findByPrimaryKey( nIdProduct );
        model.put( MARK_PRODUCT, product );
    }

    /**
     *{@inheritDoc}
     */
    public void initModelForOrder( int nIdProduct, Map<String, Object> model, HttpServletRequest request )
    {
        TicketProduct product = _homeTicketProduct.findByPrimaryKey( nIdProduct );
        model.put( MARK_PRODUCT, product );
    }

    /**
     *{@inheritDoc}
     */
    public Product getProduct( int nIdProduct )
    {
        return _homeTicketProduct.findByPrimaryKey( nIdProduct );
    }

    /**
     *{@inheritDoc}
     */
    public List<Product> findByFilter( ProductFilter filter )
    {
        return null;
    }

    /**
     *{@inheritDoc}
     */
    public ObjectDefinition getObjectDefinition(  )
    {
        if ( _objectDefinition == null )
        {
            _objectDefinition = new ObjectDefinition( CATEGORY_NAME_KEY, CATEGORY_CREATION_JSP,
                    CATEGORY_MODIFICATION_JSP, CATEGORY_DELETION_JSP );
        }

        return _objectDefinition;
    }

    /**
     *{@inheritDoc}
     */
    public boolean isAuthorizedToCreate( AdminUser user )
    {
        if ( !RBACService.isAuthorized( StockJspBean.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    StockResourceIdService.PERMISSION_CREATE, user ) )
        {
            return false;
        }

        return true;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAuthorizedToModify( AdminUser user )
    {
        if ( !RBACService.isAuthorized( StockJspBean.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    StockResourceIdService.PERMISSION_MODIFY, user ) )
        {
            return false;
        }

        return true;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isAuthorizedToDelete( AdminUser user )
    {
        if ( !RBACService.isAuthorized( StockJspBean.RESOURCE_TYPE, RBAC.WILDCARD_RESOURCES_ID,
                    StockResourceIdService.PERMISSION_DELETE, user ) )
        {
            return false;
        }

        return true;
    }

    /**
     *{@inheritDoc}
     */
    public String getMessageNotAuthorizedForCreate(  )
    {
        return MESSAGE_NOT_AUTHORIZED_TO_CREATE_TICKET_PRODUCT;
    }

    /**
     *{@inheritDoc}
     */
    public String getMessageNotAuthorizedForDelete(  )
    {
        return MESSAGE_NOT_AUTHORIZED_TO_DELETE_TICKET_PRODUCT;
    }

    /**
     *{@inheritDoc}
     */
    public String getMessageNotAuthorizedForModification(  )
    {
        return MESSAGE_NOT_AUTHORIZED_TO_MODIFY_TICKET_PRODUCT;
    }

    /**
     * {@inheritDoc}
     *
     */
    public void updateProduct( Product product )
    {
        _homeTicketProduct.update( new TicketProduct( product ) );
    }
}
