/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.web;

import fr.paris.lutece.plugins.stock.business.Category;
import fr.paris.lutece.plugins.stock.business.ICategoryHome;
import fr.paris.lutece.plugins.stock.business.Product;
import fr.paris.lutece.plugins.stock.business.order.Basket;
import fr.paris.lutece.plugins.stock.business.order.IBasketHome;
import fr.paris.lutece.plugins.stock.business.order.IItemHome;
import fr.paris.lutece.plugins.stock.business.order.Item;
import fr.paris.lutece.plugins.stock.service.GlobalProductService;
import fr.paris.lutece.plugins.stock.service.StockPlugin;
import fr.paris.lutece.plugins.stock.utils.StockUtils;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.spring.SpringContextService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;
import fr.paris.lutece.util.url.UrlItem;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 * This class provides the user interface to manage form features ( manage,
 * create, modify, remove)
 */
public class OrderJspBean extends PluginAdminPageJspBean
{
    //PARAMETERS
    private static final String PARAMETER_PRODUCT_ID = "product_id";
    private static final String PARAMETER_QUANTITY = "quantity";
    private static final String MESSAGE_NOT_ENOUGH_STOCK = "stock.order_product.not_enough_stock";

    //JSP
    private static final String JSP_MANAGE_PRODUCTS = "jsp/admin/plugins/stock/ManageProducts.jsp";
    private static final String PROPERTY_PAGE_TITLE_ORDER_OFFER = "module.stock.kiosque.order_session.title";
    private IItemHome _homeItem;
    private IBasketHome _homeBasket;
    private ICategoryHome _homeCategory;

    /**
     * Constructor
     */
    public OrderJspBean(  )
    {
        super(  );
        _homeItem = (IItemHome) SpringContextService.getPluginBean( StockPlugin.PLUGIN_NAME, "stock.itemHome" );
        _homeBasket = (IBasketHome) SpringContextService.getPluginBean( StockPlugin.PLUGIN_NAME, "stock.basketHome" );
        _homeCategory = (ICategoryHome) SpringContextService.getPluginBean( StockPlugin.PLUGIN_NAME,
                "stock.categoryHome" );
    }

    /**
     * Return management page of product
     * @param request The Http request
     * @param productClassName The class name of the product entity to order
     * @return Html management page of plugin stock
     */
    public String getOrderProduct( HttpServletRequest request, String productClassName )
    {
        setPageTitleProperty( PROPERTY_PAGE_TITLE_ORDER_OFFER );

        int nIdProduct = -1;

        nIdProduct = Integer.parseInt( request.getParameter( PARAMETER_PRODUCT_ID ) );

        Map<String, Object> model = new HashMap<String, Object>(  );

        GlobalProductService.getInstance(  ).initModelForOrder( productClassName, nIdProduct, model, request );

        String strTemplateName = GlobalProductService.getInstance(  ).getOrderTemplate( productClassName );

        HtmlTemplate template = AppTemplateService.getTemplate( strTemplateName, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Process the data capture form of a new product order
     * @param request The Http Request
     * @param productClassName The class name of the product entity to order
     * @return The Jsp URL of the process result
     */
    public String doOrderProduct( HttpServletRequest request, String productClassName )
    {
        int nIdProduct = -1;
        int nQuantity = 0;
        int nIdUser = getUser(  ).getUserId(  );

        try
        {
            nIdProduct = Integer.parseInt( request.getParameter( PARAMETER_PRODUCT_ID ) );
            nQuantity = Integer.parseInt( request.getParameter( PARAMETER_QUANTITY ) );
        }
        catch ( NumberFormatException e )
        {
            return AdminMessageService.getMessageUrl( request, StockUtils.MESSAGE_ERROR_OCCUR, AdminMessage.TYPE_STOP );
        }

        //check the quantity in stock
        Product product = GlobalProductService.getInstance(  ).getProduct( productClassName, nIdProduct );

        if ( ( product.getStockQuantity(  ) != -1 ) && ( nQuantity > product.getStockQuantity(  ) ) )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_NOT_ENOUGH_STOCK, AdminMessage.TYPE_STOP );
        }

        //update product quantity
        int newQuantity = product.getStockQuantity(  ) - nQuantity;
        product.setStockQuantity( newQuantity );
        GlobalProductService.getInstance(  ).doModifyProduct( productClassName, product, request );

        // create the basket for current user if needed
        Basket basket = null;
        List<Item> itemList = null;
        basket = _homeBasket.findValidatedByUserId( nIdUser );

        if ( basket == null )
        {
            basket = new Basket(  );
            itemList = new ArrayList<Item>(  );

            basket.setItemList( itemList );
            basket.setIdUser( nIdUser );

            _homeBasket.create( basket );
        }
        else
        {
            itemList = basket.getItemList(  );
        }

        //create the item to add to the basket
        Item item = new Item(  );
        item.setBasket( basket );
        item.setIdProduct( nIdProduct );
        item.setPrice( product.getPrice(  ) );
        item.setQuantity( nQuantity );

        Category category = _homeCategory.findByPrimaryKey( product.getCategory(  ).getIdCategory(  ) );
        item.setSummary( category.getName(  ) );
        _homeItem.create( item );

        itemList.add( item );
        _homeBasket.update( basket );

        return getJspManageProduct( request, product );
    }

    /**
     * Returns url of the jsp manage product
     * @param request The HTTP request
     * @param product The product to manage
     * @return url of the jsp manage product
     */
    public String getJspManageProduct( HttpServletRequest request, Product product )
    {
        UrlItem urlItem = new UrlItem( AppPathService.getBaseUrl( request ) + JSP_MANAGE_PRODUCTS );

        urlItem.addParameter( StockJspBean.PARAMETER_CATEGORY_ID, product.getCategory(  ).getIdCategory(  ) );

        return urlItem.getUrl(  );
    }
}
