/*
 * Copyright (c) 2002-2011, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.stock.web;

import fr.paris.lutece.plugins.stock.business.Category;
import fr.paris.lutece.plugins.stock.business.CategoryFilter;
import fr.paris.lutece.plugins.stock.business.ICategoryHome;
import fr.paris.lutece.plugins.stock.business.IProviderHome;
import fr.paris.lutece.plugins.stock.business.ObjectDefinition;
import fr.paris.lutece.plugins.stock.business.Product;
import fr.paris.lutece.plugins.stock.business.Provider;
import fr.paris.lutece.plugins.stock.service.GlobalCategoryService;
import fr.paris.lutece.plugins.stock.service.GlobalProductService;
import fr.paris.lutece.plugins.stock.service.StockPlugin;
import fr.paris.lutece.plugins.stock.utils.StockUtils;
import fr.paris.lutece.plugins.stock.utils.constants.StockConstants;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.spring.SpringContextService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.portal.web.constants.Messages;
import fr.paris.lutece.portal.web.constants.Parameters;
import fr.paris.lutece.portal.web.util.LocalizedPaginator;
import fr.paris.lutece.util.html.HtmlTemplate;
import fr.paris.lutece.util.html.Paginator;
import fr.paris.lutece.util.url.UrlItem;

import org.apache.commons.lang.StringUtils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 * This class provides the user interface to manage form features ( manage,
 * create, modify, remove)
 */
public class StockJspBean extends PluginAdminPageJspBean
{
    public static final String RIGHT_MANAGE_STOCK = "STOCK_MANAGEMENT";
    public static final String RESOURCE_TYPE = "STOCK";
    public static final String PARAMETER_CATEGORY_ID = "category_id";
    public static final String PAGE_RETURN = "page_return";
    public static final String PARAMETER_CATEGORY_ACTION_ID = "category_action_id";
    public static final String PARAMETER_PROVIDER_NAME = "provider_name";
    public static final String PARAMETER_PROVIDER_ID = "provider_id";

    //PROPERTIES
    protected static final String PROPERTY_DEFAULT_SESSION_PER_PAGE = "stock.itemsPerPage";

    //TEMPLATES
    private static final String TEMPLATE_MANAGE_PLUGIN_STOCK = "admin/plugins/stock/manage_stock.html";
    private static final String TEMPLATE_MANAGE_PRODUCT = "admin/plugins/stock/manage_products.html";
    private static final String TEMPLATE_MANAGE_CATEGORIES = "admin/plugins/stock/manage_categories.html";
    private static final String TEMPLATE_MANAGE_CATEGORIES_TYPES = "admin/plugins/stock/list_categories_types.html";
    private static final String TEMPLATE_MANAGE_PRODUCTS_TYPES = "admin/plugins/stock/list_products_types.html";
    private static final String JSP_LIST_OFFER_PARTNER = "jsp/admin/plugins/stock/modules/kiosque/ListOfferPartner.jsp";

    //JSP
    private static final String JSP_MANAGE_PRODUCTS = "jsp/admin/plugins/stock/ManageProducts.jsp";
    private static final String JSP_MANAGE_CATEGORIES = "jsp/admin/plugins/stock/ManageCategories.jsp";

    // PAGE TITLES
    private static final String PROPERTY_PAGE_TITLE_CREATE_CATEGORY = "stock.create_category.title";
    private static final String PROPERTY_PAGE_TITLE_LIST_OFFER_PROVIDER = "stock.list_offer_provider.title";
    private static final String PROPERTY_PAGE_TITLE_MODIFY_CATEGORY = "stock.modify_category.title";
    private static final String PROPERTY_PAGE_TITLE_DUPLICATE_CATEGORY = "stock.duplicate_category.title";
    private static final String PROPERTY_PAGE_TITLE_MANAGE_CATEGORY = "stock.manage_stock.category_management";
    private static final String PROPERTY_PAGE_TITLE_MANAGE_PRODUCT = "stock.manage_stock.product_management";
    private static final String PROPERTY_PAGE_TITLE_MANAGE_CATEGORY_CREATION = "stock.manage_category_creation.pageTitle";
    private static final String PROPERTY_PAGE_TITLE_MANAGE_PRODUCT_CREATION = "stock.manage_product_creation.pageTitle";

    //PARAMETERS
    private static final String PARAMETER_PRODUCT_ID = "product_id";
    private static final String PARAMETER_CATEGORY_NAME = "category_name";
    private static final String PARAMETER_CATEGORY_ACTIVE = "category_active";
    private static final String PARAMETER_CATEGORY_ROOT = "category_root";

    //MARKS
    private static final String MARK_LIST_CATEGORIES = "list_categories";
    private static final String MARK_LIST_FINAL_CATEGORIES = "list_final_categories";
    private static final String MARK_SELECTED_CATEGORY = "selected_category";
    private static final String MARK_PAGINATOR = "paginator";
    private static final String MARK_NB_ITEMS_PER_PAGE = "nb_items_per_page";
    private static final String MARK_LIST_OBJECT_DEFINITION = "list_object_definition";
    private static final String MARK_PRODUCT_FILTER = "filter";
    private static final String MARK_LOCALE = "locale";
    private static final String MARK_ERROR = "error";

    //MESSAGES
    private static final String MESSAGE_CONFIRMATION_DELETE_PRODUCT = "stock.manage_product.delete_confirmation";
    private static final String MESSAGE_CONFIRMATION_DELETE_CATEGORY = "stock.manage_category.delete_confirmation";
    private static final String MESSAGE_ERROR_NOT_DIGIT_QUANTITY = "stock.message.error.notDigitQuantity";

    //MEMBERS VARIABLES
    private ICategoryHome _homeCategory;
    private IProviderHome _homeProvider;
    private int _nDefaultItemsPerPage;
    private String _strCurrentPageIndex;
    private int _nItemsPerPage;
    private CategoryFilter _categoryFilter = new CategoryFilter(  );

    /**
     * Constructor
     */
    public StockJspBean(  )
    {
        super(  );

        _homeCategory = (ICategoryHome) SpringContextService.getPluginBean( StockPlugin.PLUGIN_NAME,
                "stock.categoryHome" );
        _homeProvider = (IProviderHome) SpringContextService.getPluginBean( StockPlugin.PLUGIN_NAME,
                "stock.providerHome" );
    }

    /**
    * Return management page of plugin stock
    * @param request The Http request
    * @return Html management page of plugin stock
    */
    public String getManageStock( HttpServletRequest request )
    {
        setPageTitleProperty( StockConstants.EMPTY_STRING );

        Map<String, String> model = new HashMap<String, String>(  );

        HtmlTemplate t = AppTemplateService.getTemplate( TEMPLATE_MANAGE_PLUGIN_STOCK, getLocale(  ), model );

        return getAdminPage( t.getHtml(  ) );
    }

    /**
     * Return the category id contained in the request
     * @param request the request
     * @return category id
     */
    protected static Integer getIdCategory( HttpServletRequest request )
    {
        String strIdCategory = request.getParameter( PARAMETER_CATEGORY_ID );

        if ( strIdCategory != null )
        {
            return Integer.parseInt( strIdCategory );
        }

        return null;
    }

    /**
     * Get the filter contained in the request
     * @param request the request
     * @return the category filter
     */
    private CategoryFilter getCategoryFilter( HttpServletRequest request )
    {
        if ( request.getParameter( StockConstants.PARAMETER_FILTER ) != null )
        {
            CategoryFilter filter = new CategoryFilter(  );
            filter.setName( request.getParameter( StockConstants.PARAMETER_FILTER_NAME ) );

            filter.setIdCategory( getIdCategory( request ) );

            _categoryFilter = filter;
        }

        // SORT
        String strSortedAttributeName = request.getParameter( Parameters.SORTED_ATTRIBUTE_NAME );

        if ( strSortedAttributeName != null )
        {
            _categoryFilter.setOrder( strSortedAttributeName );

            String strAscSort = request.getParameter( Parameters.SORTED_ASC );

            boolean bIsAscSort = Boolean.parseBoolean( strAscSort );
            _categoryFilter.setOrderAsc( bIsAscSort );
        }

        return _categoryFilter;
    }

    /**
     * Return management page of product
     * @param request The Http request
     * @return Html management page of products
     */
    public String getManageProducts( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_TITLE_MANAGE_PRODUCT );

        CategoryFilter filter = getCategoryFilter( request );

        _strCurrentPageIndex = Paginator.getPageIndex( request, Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );
        _nDefaultItemsPerPage = AppPropertiesService.getPropertyInt( PROPERTY_DEFAULT_SESSION_PER_PAGE, 50 );
        _nItemsPerPage = Paginator.getItemsPerPage( request, Paginator.PARAMETER_ITEMS_PER_PAGE, _nItemsPerPage,
                _nDefaultItemsPerPage );

        Map<String, Object> model = new HashMap<String, Object>(  );

        model.put( MARK_PRODUCT_FILTER, filter );

        // possibly displayed categories
        List<Category> allCategories = null;
        Category selectedCategory = null;
        List<Category> finalCategories = null;

        if ( !filter.hasFilter(  ) )
        {
            //categories with children
            allCategories = _homeCategory.findAllFirstLevelWithChildren(  );
        }
        else
        {
            if ( filter.getIdCategory(  ) != null )
            {
                selectedCategory = _homeCategory.findByPrimarykeyWithChildren( filter.getIdCategory(  ) );

                CategoryFilter filterSubCategories = new CategoryFilter(  );
                filterSubCategories.setParentCategory( filter.getIdCategory(  ) );
                filterSubCategories.setOrder( filter.getOrder(  ) );
                filterSubCategories.setOrderAsc( filter.isOrderAsc(  ) );
                allCategories = _homeCategory.findByFilterWithChildren( filterSubCategories );

                //categories without children and containing a product
            }
            else
            {
                // get list products
                filter.setProductNull( Boolean.FALSE );
                finalCategories = GlobalCategoryService.getInstance(  ).findByFilter( Category.class.getName(  ), filter );
            }
        }

        // category mode
        if ( allCategories != null )
        {
            //categories with children
            List<Category> categories = new ArrayList<Category>(  );
            finalCategories = new ArrayList<Category>(  );

            for ( Category subCategory : allCategories )
            {
                if ( ( subCategory.getChildrenList(  ) != null ) && ( subCategory.getChildrenList(  ).size(  ) > 0 ) &&
                        ( ( subCategory.getProductSet(  ) == null ) || ( subCategory.getProductSet(  ).size(  ) == 0 ) ) )
                {
                    categories.add( subCategory );
                }
                else if ( ( subCategory.getProductSet(  ) != null ) && ( subCategory.getProductSet(  ).size(  ) > 0 ) )
                {
                    finalCategories.add( subCategory );
                }
            }

            model.put( MARK_LIST_CATEGORIES, categories );
            model.put( MARK_SELECTED_CATEGORY, selectedCategory );
        }

        Paginator<Category> paginator = new LocalizedPaginator<Category>( finalCategories, _nItemsPerPage,
                getJspManageProduct( request ), Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex, getLocale(  ) );
        model.put( MARK_NB_ITEMS_PER_PAGE, "" + _nItemsPerPage );
        model.put( MARK_PAGINATOR, paginator );
        model.put( MARK_LIST_FINAL_CATEGORIES, paginator.getPageItems(  ) );

        HtmlTemplate t = AppTemplateService.getTemplate( TEMPLATE_MANAGE_PRODUCT, getLocale(  ), model );

        return getAdminPage( t.getHtml(  ) );
    }

    /**
     * Return the Html code for choosing the category type to create
     * @param request The Http request
     * @return the Html code for choosing the category type to create
     */
    public String getManageCreationProduct( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_TITLE_MANAGE_PRODUCT_CREATION );

        List<ObjectDefinition> listObjectDefinitions = GlobalProductService.getInstance(  )
                                                                           .getAllRegisteredObjectDefinition( getUser(  ) );
        Integer idCategoryParent = getIdCategory( request );
        int nIdCategoryParent = -1;

        if ( idCategoryParent != null )
        {
            nIdCategoryParent = idCategoryParent.intValue(  );
        }

        HashMap<String, Object> model = new HashMap<String, Object>(  );
        model.put( MARK_LIST_OBJECT_DEFINITION, listObjectDefinitions );
        model.put( PARAMETER_CATEGORY_ID, nIdCategoryParent );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_MANAGE_PRODUCTS_TYPES, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Returns the form to create a product
     *
     * @param request The Http request
     * @param strProductClassName The class used for the product
     * @return the html code of the product form
     */
    public String getCreateProduct( HttpServletRequest request, String strProductClassName )
    {
        GlobalProductService serviceProduct = GlobalProductService.getInstance(  );

        setPageTitleProperty( StockConstants.EMPTY_STRING );

        Integer idCategoryParent = getIdCategory( request );
        int idLocalRootCategoryValue = -1;

        if ( idCategoryParent != null )
        {
            idLocalRootCategoryValue = idCategoryParent.intValue(  );
        }

        String strIdProvider = request.getParameter( ProviderJspBean.PARAMETER_PROVIDER_ID );

        if ( StringUtils.isBlank( strIdProvider ) )
        {
            return AdminMessageService.getMessageUrl( request, StockConstants.MESSAGE_ERROR_OCCUR,
                AdminMessage.TYPE_STOP );
        }

        Map<String, Object> model = new HashMap<String, Object>(  );

        if ( !serviceProduct.isAuthorizedToCreate( getUser(  ), strProductClassName ) )
        {
            model.put( MARK_ERROR, serviceProduct.getMessageNotAuthorizedForCreate( strProductClassName ) );
        }
        else
        {
            model.put( PARAMETER_CATEGORY_ID, idLocalRootCategoryValue );
            model.put( ProviderJspBean.PARAMETER_PROVIDER_ID, strIdProvider );

            Collection<Category> listCategories = _homeCategory.findAll(  );

            model.put( MARK_LIST_CATEGORIES, listCategories );
        }

        String strTemplateName = serviceProduct.getCreateTemplate( strProductClassName );

        HtmlTemplate template = AppTemplateService.getTemplate( strTemplateName, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Process the data capture form of a new product
     *
     * @param request The Http Request
     * @param strProductClassName The class used for the product
     * @return The Jsp URL of the process result
     */
    public String doCreateProduct( HttpServletRequest request, String strProductClassName )
    {
        String strError = null;
        String strCancel = request.getParameter( StockConstants.PARAMETER_BUTTON_CANCEL );

        if ( !GlobalProductService.getInstance(  ).isAuthorizedToCreate( getUser(  ), strProductClassName ) ||
                !StringUtils.isBlank( strCancel ) )
        {
            return getJspManageCategories( request );
        }

        Product product = new Product(  );

        String strIdCategory = request.getParameter( PARAMETER_CATEGORY_ID );
        String strIdProvider = request.getParameter( ProviderJspBean.PARAMETER_PROVIDER_ID );

        if ( StringUtils.isBlank( strIdCategory ) || StringUtils.isBlank( strIdProvider ) )
        {
            return AdminMessageService.getMessageUrl( request, StockUtils.MESSAGE_ERROR_OCCUR, AdminMessage.TYPE_STOP );
        }

        try
        {
            Category category = _homeCategory.findByPrimaryKey( Integer.parseInt( strIdCategory ) );

            if ( category != null )
            {
                product.setCategory( category );
            }

            Provider provider = _homeProvider.findByIdWithProducts( Integer.valueOf( strIdProvider ) );

            if ( provider != null )
            {
                if ( !provider.getProducts(  ).contains( category ) )
                {
                    provider.getProducts(  ).add( category );
                }

                _homeProvider.update( provider );
                category.setProvider( provider );
                _homeCategory.update( category );
            }

            String strPrice = request.getParameter( StockConstants.PARAMETER_PRODUCT_PRICE );

            if ( ( strPrice != null ) && !strPrice.equals( StockConstants.EMPTY_STRING ) )
            {
                product.setPrice( Float.parseFloat( strPrice ) );
            }
            else
            {
                product.setPrice( 0 );
            }

            String strQuantity = request.getParameter( StockConstants.PARAMETER_PRODUCT_QUANTITY );

            if ( ( strQuantity != null ) && !strQuantity.equals( StockConstants.EMPTY_STRING ) )
            {
                int nQuantity = StockConstants.UNDEFINED;

                try
                {
                    nQuantity = Integer.parseInt( strQuantity );
                }
                catch ( Exception e )
                {
                    return AdminMessageService.getMessageUrl( request, MESSAGE_ERROR_NOT_DIGIT_QUANTITY,
                        AdminMessage.TYPE_STOP );
                }

                product.setStockQuantity( nQuantity );
            }
            else
            {
                product.setStockQuantity( 0 );
            }

            strError = GlobalProductService.getInstance(  ).doCreateProduct( strProductClassName, product, request );

            if ( strError != null )
            {
                return AdminMessageService.getMessageUrl( request, strError, AdminMessage.TYPE_STOP );
            }
        }
        catch ( NumberFormatException e )
        {
            return AdminMessageService.getMessageUrl( request, StockUtils.MESSAGE_ERROR_OCCUR, AdminMessage.TYPE_STOP );
        }

        return getReturnJspAfterCreateProduct( request, product );
    }

    /**
     * Returns the form to modify a product
     *
     * @param request The Http request
     * @param strProductClassName The class used for the product
     * @return the html code of the product form
     */
    public String getModifyProduct( HttpServletRequest request, String strProductClassName )
    {
        GlobalProductService serviceProduct = GlobalProductService.getInstance(  );

        setPageTitleProperty( StockConstants.EMPTY_STRING );

        int nIdProduct = -1;

        try
        {
            nIdProduct = Integer.parseInt( request.getParameter( PARAMETER_PRODUCT_ID ) );
        }
        catch ( NumberFormatException e )
        {
            getManageProducts( request );
        }

        //For categories selection
        Collection<Category> listCategories = _homeCategory.findAll(  );

        Integer idCategoryParent = getIdCategory( request );
        int idLocalRootCategoryValue = -1;

        if ( idCategoryParent != null )
        {
            idLocalRootCategoryValue = idCategoryParent.intValue(  );
        }

        Map<String, Object> model = new HashMap<String, Object>(  );

        if ( !serviceProduct.isAuthorizedToModify( getUser(  ), strProductClassName ) )
        {
            model.put( MARK_ERROR, serviceProduct.getMessageNotAuthorizedForModification( strProductClassName ) );
        }
        else
        {
            model.put( PARAMETER_CATEGORY_ID, idLocalRootCategoryValue );
            //to initialize the product object
            GlobalProductService.getInstance(  )
                                .initModelForModification( strProductClassName, nIdProduct, model, request );

            model.put( MARK_LIST_CATEGORIES, listCategories );
        }

        String strTemplateName = serviceProduct.getModifyTemplate( strProductClassName );

        HtmlTemplate template = AppTemplateService.getTemplate( strTemplateName, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Process the data capture form to modify a product
     *
     * @param request The Http Request
     * @param strProductClassName The class used for the product
     * @return The Jsp URL of the process result
     */
    public String doModifyProduct( HttpServletRequest request, String strProductClassName )
    {
        String strCancel = request.getParameter( StockConstants.PARAMETER_BUTTON_CANCEL );

        GlobalProductService globalProductService = GlobalProductService.getInstance(  );

        if ( !globalProductService.isAuthorizedToModify( getUser(  ), strProductClassName ) ||
                !StringUtils.isBlank( strCancel ) )
        {
            return getJspManageProduct( request );
        }

        String strError = null;
        String strIdProduct = request.getParameter( PARAMETER_PRODUCT_ID );
        Product product = null;

        synchronized ( globalProductService )
        {
            product = globalProductService.getProduct( strProductClassName, Integer.parseInt( strIdProduct ) );

            if ( product != null )
            {
                strError = globalProductService.doModifyProduct( strProductClassName, product, request );

                if ( StringUtils.isNotBlank( strError ) )
                {
                    return strError;
                }
            }
            else
            {
                return AdminMessageService.getMessageUrl( request, StockUtils.MESSAGE_ERROR_OCCUR,
                    AdminMessage.TYPE_STOP );
            }
        }

        return getReturnJspAfterModifyProduct( request, product );
    }

    /**
     * Returns the confirmation message to delete a product
     *
     * @param request The Http request
     * @param strProductClassName The class used for the product
     * @return the html code message
     */
    public String getDeleteProduct( HttpServletRequest request, String strProductClassName )
    {
        GlobalProductService serviceProduct = GlobalProductService.getInstance(  );

        if ( !serviceProduct.isAuthorizedToDelete( getUser(  ), strProductClassName ) )
        {
            return AdminMessageService.getMessageUrl( request,
                serviceProduct.getMessageNotAuthorizedForDelete( strProductClassName ), AdminMessage.TYPE_STOP );
        }

        String strProductId = request.getParameter( PARAMETER_PRODUCT_ID );

        int nIdProduct;

        try
        {
            nIdProduct = Integer.parseInt( strProductId );
        }
        catch ( NumberFormatException e )
        {
            AppLogService.debug( e );

            return AdminMessageService.getMessageUrl( request, StockConstants.MESSAGE_ERROR_OCCUR,
                AdminMessage.TYPE_STOP );
        }

        UrlItem url = new UrlItem( GlobalProductService.getInstance(  ).getDeleteJsp( strProductClassName ) );

        Map<String, Object> urlParam = new HashMap<String, Object>(  );
        getDeleteUrlParameters( nIdProduct, url, urlParam, request );

        return AdminMessageService.getMessageUrl( request, getMessageDeleteProductConfirmation(  ), url.getUrl(  ),
            AdminMessage.TYPE_CONFIRMATION, urlParam );
    }

    /**
     * Deletes a product
     *
     * @param request The Http request
     * @param strProductClassName The class used for the product
     * @return the html code message
     */
    public String doDeleteProduct( HttpServletRequest request, String strProductClassName )
    {
        String strProductId = request.getParameter( PARAMETER_PRODUCT_ID );

        int nIdProduct;

        try
        {
            nIdProduct = Integer.parseInt( strProductId );
        }
        catch ( NumberFormatException e )
        {
            AppLogService.debug( e );

            return AdminMessageService.getMessageUrl( request, StockUtils.MESSAGE_ERROR_OCCUR, AdminMessage.TYPE_STOP );
        }

        GlobalProductService.getInstance(  ).doDeleteProduct( strProductClassName, nIdProduct );

        return getJspManageProduct( request );
    }

    /**
     * Return management page of categories
     * @param request The Http request
     * @return Html management page of categories
     */
    public String getManageCategories( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_TITLE_MANAGE_CATEGORY );

        _strCurrentPageIndex = Paginator.getPageIndex( request, Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );
        _nDefaultItemsPerPage = AppPropertiesService.getPropertyInt( PROPERTY_DEFAULT_SESSION_PER_PAGE, 50 );
        _nItemsPerPage = Paginator.getItemsPerPage( request, Paginator.PARAMETER_ITEMS_PER_PAGE, _nItemsPerPage,
                _nDefaultItemsPerPage );

        Map<String, Object> model = new HashMap<String, Object>(  );

        Integer idLocalCategoryRoot = getIdCategory( request );
        List<Category> listCategories = null;
        int idLocalCategoryRootValue = -1;

        if ( ( idLocalCategoryRoot == null ) || ( idLocalCategoryRoot.intValue(  ) == StockConstants.UNDEFINED ) )
        {
            listCategories = _homeCategory.findAllFirstLevelWithChildren(  );
        }
        else
        {
            idLocalCategoryRootValue = idLocalCategoryRoot.intValue(  );
            listCategories = _homeCategory.findAllChildrenWithChildren( idLocalCategoryRootValue );
        }

        Paginator<Category> paginator = new LocalizedPaginator<Category>( listCategories, _nItemsPerPage,
                getJspManageCategories( request ), Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex, getLocale(  ) );

        Category category = _homeCategory.findByPrimaryKey( idLocalCategoryRootValue );
        model.put( MARK_PAGINATOR, paginator );
        model.put( MARK_LIST_CATEGORIES, paginator.getPageItems(  ) );
        model.put( MARK_NB_ITEMS_PER_PAGE, "" + _nItemsPerPage );
        model.put( PARAMETER_CATEGORY_ROOT, category );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_MANAGE_CATEGORIES, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Returns the Html code for choosing the category type to create
     * @param request The Http request
     * @return the Html code for choosing the category type to create
     */
    public String getManageCreationCategory( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_TITLE_MANAGE_CATEGORY_CREATION );

        List<ObjectDefinition> listObjectDefinitions = GlobalCategoryService.getInstance(  )
                                                                            .getAllRegisteredObjectDefinition(  );
        Integer idLocalRootCategory = getIdCategory( request );
        int idLocalRootCategoryValue = -1;

        if ( idLocalRootCategory != null )
        {
            idLocalRootCategoryValue = idLocalRootCategory.intValue(  );
        }

        HashMap<String, Object> model = new HashMap<String, Object>(  );
        model.put( MARK_LIST_OBJECT_DEFINITION, listObjectDefinitions );
        model.put( PARAMETER_CATEGORY_ID, idLocalRootCategoryValue );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_MANAGE_CATEGORIES_TYPES, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Returns the form to create a category
     *
     * @param request The Http request
     * @param strCategoryClassName The class used for the category
     * @return the html code of the category form
     */
    public String getCreateCategory( HttpServletRequest request, String strCategoryClassName )
    {
        GlobalCategoryService serviceCategory = GlobalCategoryService.getInstance(  );

        setPageTitleProperty( getCreatePageTitle(  ) );

        Integer idLocalRootCategory = getIdCategory( request );

        if ( idLocalRootCategory == null )
        {
            idLocalRootCategory = StockConstants.UNDEFINED;
        }

        String strIdProvider = request.getParameter( ProviderJspBean.PARAMETER_PROVIDER_ID );
        int idProvider = Integer.parseInt( strIdProvider );
        Provider provider = _homeProvider.findByIdWithProducts( idProvider );
        String strNameProvider = provider.getName( );

        Map<String, Object> model = new HashMap<String, Object>(  );

        if ( !serviceCategory.isAuthorizedToCreate( getUser(  ), strCategoryClassName ) )
        {
            model.put( MARK_ERROR, serviceCategory.getMessageNotAuthorizedForCreate( strCategoryClassName ) );
        }
        else
        {
            model.put( PARAMETER_CATEGORY_ID, idLocalRootCategory.intValue(  ) );
            model.put( ProviderJspBean.PARAMETER_PROVIDER_ID, strIdProvider );
            model.put( PARAMETER_PROVIDER_NAME, strNameProvider );

            GlobalCategoryService.getInstance(  ).initModelForCreation( strCategoryClassName, model );
        }

        String strTemplateName = serviceCategory.getCreateTemplate( strCategoryClassName );

        HtmlTemplate template = AppTemplateService.getTemplate( strTemplateName, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Returns the form to duplicate a category
     * 
     * @param request The Http request
     * @param strCategoryClassName The class used for the category
     * @return the html code of the category form
     */
    public String getDuplicateCategory( HttpServletRequest request, String strCategoryClassName )
    {
        GlobalCategoryService serviceCategory = GlobalCategoryService.getInstance( );

        setPageTitleProperty( getDuplicatePageTitle( ) );

        Integer idLocalRootCategory = getIdCategory( request );

        if ( idLocalRootCategory == -1 )
        {
            idLocalRootCategory = Integer.parseInt( request.getParameter( PARAMETER_CATEGORY_ACTION_ID ) );
        }

        if ( idLocalRootCategory == null )
        {
            idLocalRootCategory = StockConstants.UNDEFINED;
        }

        String strIdProvider = request.getParameter( ProviderJspBean.PARAMETER_PROVIDER_ID );
        int idProvider = Integer.parseInt( strIdProvider );
        Provider provider = _homeProvider.findByIdWithProducts( idProvider );
        String strNameProvider = provider.getName( );
        
        String strCategoryName = request.getParameter( PARAMETER_CATEGORY_NAME );

        Map<String, Object> model = new HashMap<String, Object>( );

        if ( !serviceCategory.isAuthorizedToCreate( getUser( ), strCategoryClassName ) )
        {
            model.put( MARK_ERROR, serviceCategory.getMessageNotAuthorizedForCreate( strCategoryClassName ) );
        }
        else
        {
            model.put( PARAMETER_CATEGORY_ID, idLocalRootCategory.intValue( ) );
            model.put( ProviderJspBean.PARAMETER_PROVIDER_ID, strIdProvider );
            model.put( PARAMETER_PROVIDER_NAME, strNameProvider );
            model.put( PARAMETER_CATEGORY_NAME, strCategoryName );

            GlobalCategoryService.getInstance( ).initModelForDuplication( idLocalRootCategory, strCategoryClassName,
                    model );
        }

        String strTemplateName = serviceCategory.getDuplicateTemplate( strCategoryClassName );

        HtmlTemplate template = AppTemplateService.getTemplate( strTemplateName, getLocale( ), model );

        return getAdminPage( template.getHtml( ) );
    }

    /**
     * Process the data capture form of a new category
     *
     * @param request The Http Request
     * @param strClassName The class used for the category
     * @return The Jsp URL of the process result
     */
    public String doCreateCategory( HttpServletRequest request, String strClassName )
    {
        String strCancel = request.getParameter( StockConstants.PARAMETER_BUTTON_CANCEL );

        if ( !GlobalCategoryService.getInstance(  ).isAuthorizedToCreate( getUser(  ), strClassName ) ||
                !StringUtils.isBlank( strCancel ) )
        {
            return getJspManageCategories( request );
        }

        String strCategoryName = request.getParameter( PARAMETER_CATEGORY_NAME );

        // Controls mandatory fields
        if ( ( strCategoryName == null ) || strCategoryName.equals( StockConstants.EMPTY_STRING ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
        }

        Category category = new Category(  );
        category.setName( strCategoryName );

        if ( request.getParameter( PARAMETER_CATEGORY_ACTIVE ) != null )
        {
            category.setActive( true );
        }
        else
        {
            category.setActive( false );
        }

        String error = null;

        error = GlobalCategoryService.getInstance( ).doCreateCategory( strClassName, category, request );

        if ( error != null )
        {
            return AdminMessageService.getMessageUrl( request, error, AdminMessage.TYPE_STOP );
        }

        return getReturnJspAfterCreateCategory( request, category );
    }

    /**
     * Returns the form to modify a category
     *
     * @param request The Http request
     * @param strCategoryClassName The class used for the product
     * @return the html code of the category form
     */
    public String getModifyCategory( HttpServletRequest request, String strCategoryClassName )
    {
        GlobalCategoryService serviceCategory = GlobalCategoryService.getInstance(  );

        setPageTitleProperty( getModifyPageTitle(  ) );

        Integer nIdCategoryToModify = -1;

        try
        {
            nIdCategoryToModify = Integer.parseInt( request.getParameter( PARAMETER_CATEGORY_ACTION_ID ) );
        }
        catch ( NumberFormatException e )
        {
            return getManageCategories( request );
        }

        // Controls mandatory fields
        if ( nIdCategoryToModify == -1 )
        {
            return getManageCategories( request );
        }

        Integer idCategory = getIdCategory( request );

        if ( idCategory == null )
        {
            return getManageCategories( request );
        }

        int nIdCategory = idCategory;

        Map<String, Object> model = new HashMap<String, Object>(  );

        if ( !serviceCategory.isAuthorizedToModify( getUser(  ), strCategoryClassName ) )
        {
            model.put( MARK_ERROR, serviceCategory.getMessageNotAuthorizedForModification( strCategoryClassName ) );
        }
        else
        {
            // Add the id of the category to display after the modification
            model.put( PAGE_RETURN, request.getParameter( PAGE_RETURN ) );
            model.put( PARAMETER_CATEGORY_ID, nIdCategory );
            model.put( PARAMETER_CATEGORY_ACTION_ID, nIdCategoryToModify );
            model.put( MARK_LOCALE, request.getLocale(  ).getLanguage(  ) );

            //to initialize the product object
            GlobalCategoryService.getInstance(  )
                                 .initModelForModification( strCategoryClassName, nIdCategoryToModify, model );
        }

        String strTemplateName = GlobalCategoryService.getInstance(  ).getModifyTemplate( strCategoryClassName );

        HtmlTemplate template = AppTemplateService.getTemplate( strTemplateName, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Process the data capture form to modify a category
     *
     * @param request The Http Request
     * @param strCategoryClassName The class used for the category
     * @return The Jsp URL of the process result
     */
    public String doModifyCategory( HttpServletRequest request, String strCategoryClassName )
    {
        String strCancel = request.getParameter( StockConstants.PARAMETER_BUTTON_CANCEL );

        if ( !GlobalCategoryService.getInstance(  ).isAuthorizedToModify( getUser(  ), strCategoryClassName ) ||
                !StringUtils.isBlank( strCancel ) )
        {
            if ( request.getParameter( PAGE_RETURN ) != null && "1".equals( request.getParameter( PAGE_RETURN ) ) )
            {
                return getJspListOfferPartner( request );
            }
            return getJspManageCategories( request );
        }

        String strCategoryName = request.getParameter( PARAMETER_CATEGORY_NAME );
        Integer nIdCategoryToModify = -1;

        try
        {
            nIdCategoryToModify = Integer.parseInt( request.getParameter( PARAMETER_CATEGORY_ACTION_ID ) );
        }
        catch ( NumberFormatException e )
        {
            AppLogService.debug( e );

            // nothing to do
        }

        // Controls mandatory fields
        if ( ( strCategoryName == null ) || strCategoryName.equals( StockConstants.EMPTY_STRING ) ||
                ( nIdCategoryToModify == -1 ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
        }

        Category category = GlobalCategoryService.getInstance(  ).getCategory( strCategoryClassName, nIdCategoryToModify );
        category.setName( strCategoryName );

        if ( request.getParameter( PARAMETER_CATEGORY_ACTIVE ) != null )
        {
            category.setActive( true );
        }
        else
        {
            category.setActive( false );
        }

        String error = null;
        error = GlobalCategoryService.getInstance(  ).doModifyCategory( strCategoryClassName, category, request );

        if ( error != null )
        {
            return AdminMessageService.getMessageUrl( request, error, AdminMessage.TYPE_STOP );
        }

        return getJspManageCategories( request );
    }

    protected String getJspListOfferPartner( HttpServletRequest request )
    {
        UrlItem urlItem = new UrlItem( AppPathService.getBaseUrl( request ) + JSP_LIST_OFFER_PARTNER );
        String strIdProvider = request.getParameter( PARAMETER_PROVIDER_ID );

        if ( strIdProvider != null )
        {
            urlItem.addParameter( PARAMETER_PROVIDER_ID, strIdProvider );
        }

        return urlItem.getUrl( );
    }

    /**
     * Returns the confirmation message to delete a category
     *
     * @param request The Http request
     * @param strCategoryClassName The class used for the category
     * @return the html code message
     */
    public String getDeleteCategory( HttpServletRequest request, String strCategoryClassName )
    {
        GlobalCategoryService serviceCategory = GlobalCategoryService.getInstance(  );

        if ( !serviceCategory.isAuthorizedToDelete( getUser(  ), strCategoryClassName ) )
        {
            return AdminMessageService.getMessageUrl( request,
                serviceCategory.getMessageNotAuthorizedForDelete( strCategoryClassName ), AdminMessage.TYPE_STOP );
        }

        Integer idCategory = getIdCategory( request );

        if ( idCategory == null )
        {
            return AdminMessageService.getMessageUrl( request, StockConstants.MESSAGE_ERROR_OCCUR,
                AdminMessage.TYPE_STOP );
        }

        int nIdCategory = idCategory;

        UrlItem url = new UrlItem( serviceCategory.getDeleteJsp( strCategoryClassName ) );

        Integer nIdCategoryToDelete = -1;

        try
        {
            nIdCategoryToDelete = Integer.parseInt( request.getParameter( PARAMETER_CATEGORY_ACTION_ID ) );
        }
        catch ( NumberFormatException e )
        {
            AppLogService.debug( e );

            // nothing to do
        }

        Map<String, Object> urlParam = new HashMap<String, Object>(  );
        urlParam.put( PARAMETER_CATEGORY_ID, nIdCategory );
        urlParam.put( PARAMETER_CATEGORY_ACTION_ID, nIdCategoryToDelete );

        return AdminMessageService.getMessageUrl( request, getMessageDeleteCategoryConfirmation(  ), url.getUrl(  ),
            AdminMessage.TYPE_CONFIRMATION, urlParam );
    }

    /**
    * Deletes a category
    *
    * @param request The Http request
    * @param strCategoryClassName The class used for the category
    * @return the html code message
    */
    public String doDeleteCategory( HttpServletRequest request, String strCategoryClassName )
    {
        Integer nIdCategoryToDelete = -1;

        try
        {
            nIdCategoryToDelete = Integer.parseInt( request.getParameter( PARAMETER_CATEGORY_ACTION_ID ) );
        }
        catch ( NumberFormatException e )
        {
            return AdminMessageService.getMessageUrl( request, StockUtils.MESSAGE_ERROR_OCCUR, AdminMessage.TYPE_STOP );
        }

        // Controls mandatory fields
        if ( nIdCategoryToDelete == -1 )
        {
            return AdminMessageService.getMessageUrl( request, StockUtils.MESSAGE_ERROR_OCCUR, AdminMessage.TYPE_STOP );
        }

        GlobalCategoryService globalCategoryService = GlobalCategoryService.getInstance(  );
        Category categoryToDelete = globalCategoryService.getCategoryWithParent( strCategoryClassName,
                nIdCategoryToDelete );

        // Update the parent if exists 
        Category parent = categoryToDelete.getParent(  );

        if ( parent != null )
        {
            parent = _homeCategory.findByPrimarykeyWithChildren( parent.getIdCategory(  ) );
            parent.getChildrenList(  ).remove( categoryToDelete );
            globalCategoryService.doModifyCategory( strCategoryClassName, parent, request );
        }

        globalCategoryService.doDeleteCategory( strCategoryClassName, nIdCategoryToDelete );

        return getJspManageCategories( request );
    }

    /**
     * Returns url of the jsp manage category
     *
     * @param request The HTTP request
     * @return url of the jsp manage category
     */
    protected String getJspManageCategories( HttpServletRequest request )
    {
        UrlItem urlItem = new UrlItem( AppPathService.getBaseUrl( request ) + JSP_MANAGE_CATEGORIES );
        Integer nIdCategory = getIdCategory( request );

        if ( nIdCategory != null )
        {
            urlItem.addParameter( PARAMETER_CATEGORY_ID, nIdCategory );
        }

        return urlItem.getUrl(  );
    }

    /**
     * Returns url of the jsp after creating a category
     *
     * @param request The HTTP request
     * @param category the created category
     * @return url of the jsp after creating a category ( manage categories in this case)
     */
    protected String getReturnJspAfterCreateCategory( HttpServletRequest request, Category category )
    {
        return getJspManageCategories( request );
    }

    /**
     * Returns url of the jsp after creating a product
     *
     * @param request The HTTP request
     * @param product the created product
     * @return url of the jsp after creating a product ( manage categories in this case)
     */
    protected String getReturnJspAfterCreateProduct( HttpServletRequest request, Product product )
    {
        return getJspManageCategories( request );
    }

    /**
     * Returns url of the jsp after product modification
     *
     * @param request The HTTP request
     * @param product the modified product
     * @return url of the jsp after product modification ( manage categories in this case)
     */
    protected String getReturnJspAfterModifyProduct( HttpServletRequest request, Product product )
    {
        return getJspManageProduct( request );
    }

    /**
     * Returns url of the jsp manage product
     * @param request The HTTP request
     * @return url of the jsp manage product
     */
    public String getJspManageProduct( HttpServletRequest request )
    {
        UrlItem urlItem = new UrlItem( AppPathService.getBaseUrl( request ) + JSP_MANAGE_PRODUCTS );
        Integer nIdCategory = getIdCategory( request );

        if ( nIdCategory != null )
        {
            urlItem.addParameter( PARAMETER_CATEGORY_ID, nIdCategory );
        }

        return urlItem.getUrl(  );
    }

    /**
    * Returns message for delete confirmation
    *
    * @return the message
    */
    protected String getMessageDeleteCategoryConfirmation(  )
    {
        return MESSAGE_CONFIRMATION_DELETE_CATEGORY;
    }

    /**
     * Returns message for delete confirmation
     *
     * @return the message
     */
    protected String getMessageDeleteProductConfirmation(  )
    {
        return MESSAGE_CONFIRMATION_DELETE_PRODUCT;
    }

    /**
     * Return the title of the categorie creation page
     * @return the title
     */
    protected String getCreatePageTitle(  )
    {
        return PROPERTY_PAGE_TITLE_CREATE_CATEGORY;
    }

    /**
     * Return the title of the provider list offer page
     * @return
     */
    protected String getListOfferPageTitle( )
    {
        return PROPERTY_PAGE_TITLE_LIST_OFFER_PROVIDER;
    }

    /**
     * Return the title of the categorie modification page
     * @return the title
     */
    protected String getModifyPageTitle(  )
    {
        return PROPERTY_PAGE_TITLE_MODIFY_CATEGORY;
    }

    /**
     * Return the title of the categorie duplication page
     * @return the title
     */
    protected String getDuplicatePageTitle( )
    {
        return PROPERTY_PAGE_TITLE_DUPLICATE_CATEGORY;
    }

    /**
     * Add parameters to the deletion url of a product.
     * @param nIdProduct The id of the product to delete
     * @param url the url
     * @param urlParam The map where to put parameters
     * @param request the http request
     */
    protected void getDeleteUrlParameters( int nIdProduct, UrlItem url, Map<String, Object> urlParam, HttpServletRequest request )
    {
    	urlParam.put( PARAMETER_PRODUCT_ID, nIdProduct );
    }
    
}
